@@ -1,5 +1,5 @@
 # coding=utf-8
-from office365.directory.directory_object import DirectoryObject
+from office365.directory.object import DirectoryObject
 from office365.runtime.types.collections import StringCollection
 
 
