@@ -27,18 +27,11 @@ class GroupCollection(DeltaCollection):
 
         :param str group_name:
         """
-        grp_properties = GroupProfile(group_name)
-        grp_properties.securityEnabled = False
-        grp_properties.mailEnabled = True
-        grp_properties.groupTypes = ["Unified"]
-        return_type = self.context.groups.add(grp_properties)
+        params = GroupProfile(group_name)
+        params.securityEnabled = False
+        params.mailEnabled = True
+        params.groupTypes = ["Unified"]
 
-        def _group_created(resp):
-            """
-            :type resp: requests.Response
-            """
-            new_team = return_type.add_team()
-            return_type.set_property("team", new_team, False)
-
-        self.context.after_execute(_group_created)
-        return return_type
+        def _after_group_created(return_type):
+            return_type.add_team()
+        return self.context.groups.add(params).after_execute(_after_group_created)
