@@ -1,32 +1,40 @@
+import json
+
 from office365.communications.onlinemeetings.collection import OnlineMeetingCollection
 from office365.communications.presences.presence import Presence
 from office365.delta_collection import DeltaCollection
+from office365.directory.applications.roles.assignment_collection import AppRoleAssignmentCollection
 from office365.directory.authentication.authentication import Authentication
 from office365.directory.extensions.extension import Extension
+from office365.directory.identities.object_identity import ObjectIdentity
 from office365.directory.insights.office_graph import OfficeGraphInsights
-from office365.directory.licenses.assigned_plan import AssignedPlan
-from office365.directory.permissions.grants.oauth2 import OAuth2PermissionGrant
-from office365.directory.users.settings import UserSettings
-from office365.onedrive.sites.site import Site
-from office365.onenote.onenote import Onenote
-from office365.outlook.calendar.attendee_base import AttendeeBase
-from office365.outlook.calendar.calendar import Calendar
-from office365.outlook.calendar.group import CalendarGroup
-from office365.outlook.calendar.event import Event
-from office365.outlook.calendar.meeting_time_suggestions_result import MeetingTimeSuggestionsResult
-from office365.outlook.calendar.reminder import Reminder
 from office365.directory.licenses.assigned_license import AssignedLicense
+from office365.directory.licenses.assigned_plan import AssignedPlan
+from office365.directory.licenses.assignment_state import LicenseAssignmentState
+from office365.directory.licenses.details import LicenseDetails
 from office365.directory.object import DirectoryObject
 from office365.directory.object_collection import DirectoryObjectCollection
-from office365.directory.licenses.details import LicenseDetails
-from office365.directory.identities.object_identity import ObjectIdentity
+from office365.directory.permissions.grants.oauth2 import OAuth2PermissionGrant
 from office365.directory.profile_photo import ProfilePhoto
+from office365.directory.users.activities.collection import UserActivityCollection
+from office365.directory.users.settings import UserSettings
 from office365.entity_collection import EntityCollection
+from office365.intune.devices.data import DeviceAndAppManagementData
+from office365.intune.devices.managed import ManagedDevice
+from office365.intune.devices.managed_app_diagnostic_status import ManagedAppDiagnosticStatus
+from office365.onedrive.drives.drive import Drive
+from office365.onedrive.sites.site import Site
+from office365.onenote.onenote import Onenote
+from office365.outlook.calendar.attendees.base import AttendeeBase
+from office365.outlook.calendar.calendar import Calendar
+from office365.outlook.calendar.events.event import Event
+from office365.outlook.calendar.events.reminder import Reminder
+from office365.outlook.calendar.group import CalendarGroup
+from office365.outlook.calendar.meetingtimes.suggestions_result import MeetingTimeSuggestionsResult
 from office365.outlook.contacts.contact import Contact
 from office365.outlook.contacts.folder import ContactFolder
 from office365.outlook.convert_id_result import ConvertIdResult
-from office365.outlook.mail.folder import MailFolder
-from office365.onedrive.drives.drive import Drive
+from office365.outlook.mail.folders.collection import MailFolderCollection
 from office365.outlook.mail.mailbox_settings import MailboxSettings
 from office365.outlook.mail.messages.collection import MessageCollection
 from office365.outlook.mail.messages.message import Message
@@ -37,9 +45,9 @@ from office365.runtime.client_result import ClientResult
 from office365.runtime.client_value_collection import ClientValueCollection
 from office365.runtime.http.http_method import HttpMethod
 from office365.runtime.paths.entity import EntityPath
+from office365.runtime.paths.resource_path import ResourcePath
 from office365.runtime.queries.function import FunctionQuery
 from office365.runtime.queries.service_operation import ServiceOperationQuery
-from office365.runtime.paths.resource_path import ResourcePath
 from office365.runtime.types.collections import StringCollection
 from office365.teams.chats.collection import ChatCollection
 from office365.teams.collection import TeamCollection
@@ -66,6 +74,55 @@ class User(DirectoryObject):
         self.context.add_query(qry)
         return self
 
+    def assign_manager(self, user):
+        """
+        Assign a user's manager.
+
+        :param str or User or OrgContact user: User or office365.intune.organizations.contact.OrgContact
+            or identifier
+        """
+
+        def _construct_request(request):
+            """
+            :type request: office365.runtime.http.request_options.RequestOptions
+            """
+            request.method = HttpMethod.Put
+            request.set_header('Content-Type', "application/json")
+            request.set_header('Accept', "application/json")
+            request.data = json.dumps(request.data)
+
+        def _assign_manager(user_id):
+            """
+            :type user_id: str
+            """
+            payload = {
+                "@odata.id": "https://graph.microsoft.com/v1.0/users/{0}".format(user_id)
+            }
+            qry = ServiceOperationQuery(self.manager, "$ref", None, payload)
+            self.context.add_query(qry).before_query_execute(_construct_request)
+
+        if isinstance(user, User):
+            def _user_loaded():
+                _assign_manager(user.id)
+
+            user.ensure_property("id", _user_loaded)
+        else:
+            _assign_manager(user)
+        return self.manager
+
+    def remove_manager(self):
+        """Remove a user's manager."""
+        qry = ServiceOperationQuery(self.manager, "$ref")
+
+        def _construct_request(request):
+            """
+            :type request: office365.runtime.http.request_options.RequestOptions
+            """
+            request.method = HttpMethod.Delete
+
+        self.context.add_query(qry).before_query_execute(_construct_request)
+        return self
+
     def change_password(self, current_password, new_password):
         """
         Enable the user to update their password. Any user can update their password without belonging
@@ -79,7 +136,7 @@ class User(DirectoryObject):
         self.context.add_query(qry)
         return self
 
-    def send_mail(self, subject, body, to_recipients, cc_recipients=[], bcc_recipients=[], save_to_sent_items=False):
+    def send_mail(self, subject, body, to_recipients, cc_recipients=None, bcc_recipients=None, save_to_sent_items=False):
         """Send a new message on the fly
 
         :param str subject: The subject of the message.
@@ -90,20 +147,22 @@ class User(DirectoryObject):
         :param bool save_to_sent_items: Indicates whether to save the message in Sent Items. Specify it only if
             the parameter is false; default is true
         """
-        message = Message(self.context)
-        message.subject = subject
-        message.body = body
-        [message.to_recipients.add(Recipient.from_email(email)) for email in to_recipients]
-        [message.bcc_recipients.add(Recipient.from_email(email)) for email in bcc_recipients]
-        [message.cc_recipients.add(Recipient.from_email(email)) for email in cc_recipients]
+        return_type = Message(self.context)
+        return_type.subject = subject
+        return_type.body = body
+        [return_type.to_recipients.add(Recipient.from_email(email)) for email in to_recipients]
+        if bcc_recipients is not None:
+            [return_type.bcc_recipients.add(Recipient.from_email(email)) for email in bcc_recipients]
+        if cc_recipients is not None:
+            [return_type.cc_recipients.add(Recipient.from_email(email)) for email in cc_recipients]
 
         payload = {
-            "message": message,
+            "message": return_type,
             "saveToSentItems": save_to_sent_items
         }
         qry = ServiceOperationQuery(self, "sendmail", None, payload)
         self.context.add_query(qry)
-        return message
+        return return_type
 
     def export_personal_data(self, storage_location):
         """
@@ -120,9 +179,18 @@ class User(DirectoryObject):
         self.context.add_query(qry)
         return self
 
-    def find_meeting_times(self, attendees=None, location_constraint=None):
+    def export_device_and_app_management_data(self):
+        """"""
+        return_type = ClientResult(self.context, ClientValueCollection(DeviceAndAppManagementData))
+        qry = FunctionQuery(self, "exportDeviceAndAppManagementData", None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
+    def find_meeting_times(self, attendees=None, location_constraint=None, time_constraint=None,
+                           meeting_duration=None, max_candidates=None, is_organizer_optional=None,
+                           return_suggestion_reasons=None, minimum_attendee_percentage=None):
         """
-        Suggest meeting times and locations based on organizer and attendee availability, and time or location
+        Suggest meeting times and locations based on organizer and attendees availability, and time or location
         constraints specified as parameters.
 
         If findMeetingTimes cannot return any meeting suggestions, the response would indicate a reason in the
@@ -134,21 +202,42 @@ class User(DirectoryObject):
         that the suggested results may differ over time.
 
         :param list[AttendeeBase] or None attendees: A collection of attendees or resources for the meeting.
-            Since findMeetingTimes assumes that any attendee who is a person is always required, specify required
+            Since findMeetingTimes assumes that any attendees who is a person is always required, specify required
             for a person and resource for a resource in the corresponding type property. An empty collection causes
             findMeetingTimes to look for free time slots for only the organizer. Optional.
         :param office365.outlook.calendar.location_constraint.LocationConstraint or None location_constraint:
             The organizer's requirements about the meeting location, such as whether a suggestion for a meeting
             location is required, or there are specific locations only where the meeting can take place. Optional.
+        :param TimeConstraint time_constraint: Any time restrictions for a meeting, which can include the nature
+            of the meeting (activityDomain property) and possible meeting time periods (timeSlots property).
+            findMeetingTimes assumes activityDomain as work if you don't specify this parameter. Optional.
+        :param str meeting_duration: The length of the meeting, denoted in ISO8601 format.
+             For example, 1 hour is denoted as 'PT1H', where 'P' is the duration designator,
+             'T' is the time designator, and 'H' is the hour designator.
+             Use M to indicate minutes for the duration; for example, 2 hours and 30 minutes would be 'PT2H30M'.
+             If no meeting duration is specified, findMeetingTimes uses the default of 30 minutes. Optional.
+        :param int max_candidates: The maximum number of meeting time suggestions to be returned. Optional.
+        :param bool is_organizer_optional: Specify True if the organizer doesn't necessarily have to attend.
+             The default is false. Optiona
+        :param bool return_suggestion_reasons: Specify True to return a reason for each meeting suggestion in
+             the suggestionReason property. The default is false to not return that property. Optional.
+        :param float minimum_attendee_percentage: The minimum required confidence for a time slot to be returned
+            in the response. It is a % value ranging from 0 to 100. Optional.
         """
         payload = {
             "attendees": ClientValueCollection(AttendeeBase, attendees),
-            "locationConstraint": location_constraint
+            "locationConstraint": location_constraint,
+            "timeConstraint": time_constraint,
+            "meeting_duration": meeting_duration,
+            "maxCandidates": max_candidates,
+            "isOrganizerOptional": is_organizer_optional,
+            "returnSuggestionReasons": return_suggestion_reasons,
+            "minimumAttendeePercentage": minimum_attendee_percentage
         }
-        result = ClientResult(self.context, MeetingTimeSuggestionsResult())
-        qry = ServiceOperationQuery(self, "findMeetingTimes", None, payload, None, result)
+        return_type = ClientResult(self.context, MeetingTimeSuggestionsResult())
+        qry = ServiceOperationQuery(self, "findMeetingTimes", None, payload, None, return_type)
         self.context.add_query(qry)
-        return result
+        return return_type
 
     def get_calendar_view(self, start_dt, end_dt):
         """Get the occurrences, exceptions, and single instances of events in a calendar view defined by a time range,
@@ -162,7 +251,6 @@ class User(DirectoryObject):
         """
         return_type = EntityCollection(self.context, Event, ResourcePath("calendarView", self.resource_path))
         qry = ServiceOperationQuery(self, "calendarView", None, None, None, return_type)
-        self.context.add_query(qry)
 
         def _construct_request(request):
             """
@@ -171,7 +259,7 @@ class User(DirectoryObject):
             request.method = HttpMethod.Get
             request.url += "?startDateTime={0}&endDateTime={1}".format(start_dt.isoformat(), end_dt.isoformat())
 
-        self.context.before_execute(_construct_request)
+        self.context.add_query(qry).before_execute(_construct_request)
         return return_type
 
     def get_reminder_view(self, start_dt, end_dt):
@@ -192,6 +280,13 @@ class User(DirectoryObject):
         self.context.add_query(qry)
         return return_type
 
+    def get_managed_app_diagnostic_statuses(self):
+        """Gets diagnostics validation status for a given user."""
+        return_type = ClientResult(self.context, ClientValueCollection(ManagedAppDiagnosticStatus))
+        qry = FunctionQuery(self, "getManagedAppDiagnosticStatuses", None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
     def delete_object(self, permanent_delete=False):
         """
         :param permanent_delete: Permanently deletes the user from directory
@@ -262,6 +357,13 @@ class User(DirectoryObject):
         """True if the account is enabled; otherwise, false. This property is required when a user is created."""
         return self.properties.get('accountEnabled', None)
 
+    @property
+    def app_role_assignments(self):
+        """Get the apps and app roles which this user has been assigned."""
+        return self.properties.get('appRoleAssignments',
+                                   AppRoleAssignmentCollection(self.context,
+                                                               ResourcePath("appRoleAssignments", self.resource_path)))
+
     @property
     def chats(self):
         """The user's chats."""
@@ -321,6 +423,14 @@ class User(DirectoryObject):
         """
         return self.properties.get('creationType', None)
 
+    @property
+    def license_assignment_states(self):
+        """
+        State of license assignments for this user. Also indicates licenses that are directly-assigned and those
+        that the user has inherited through group memberships.
+        """
+        return self.properties.get('licenseAssignmentStates', ClientValueCollection(LicenseAssignmentState))
+
     @property
     def mail(self):
         """The SMTP address for the user, for example, "jeff@contoso.onmicrosoft.com".
@@ -345,6 +455,12 @@ class User(DirectoryObject):
         """
         return self.properties.get('identities', ClientValueCollection(ObjectIdentity))
 
+    @property
+    def activities(self):
+        """The user's activities across devices."""
+        return self.properties.get('activities',
+                                   UserActivityCollection(self.context, ResourcePath("activities", self.resource_path)))
+
     @property
     def assigned_licenses(self):
         """The licenses that are assigned to the user, including inherited (group-based) licenses. """
@@ -361,7 +477,7 @@ class User(DirectoryObject):
 
     @property
     def insights(self):
-        """"""
+        """Insights are relationships calculated using advanced analytics and machine learning techniques."""
         return self.properties.get('insights',
                                    OfficeGraphInsights(self.context, ResourcePath("insights", self.resource_path)))
 
@@ -394,8 +510,6 @@ class User(DirectoryObject):
     def mailbox_settings(self):
         """
         Get the user's mailboxSettings.
-
-        :rtype: str or None
         """
         return self.properties.get('mailboxSettings', MailboxSettings())
 
@@ -465,6 +579,13 @@ class User(DirectoryObject):
         return self.properties.get('joinedTeams',
                                    TeamCollection(self.context, ResourcePath("joinedTeams", self.resource_path)))
 
+    @property
+    def managed_devices(self):
+        """"""
+        return self.properties.get('managedDevices',
+                                   EntityCollection(self.context, ManagedDevice,
+                                                    ResourcePath("managedDevices", self.resource_path)))
+
     @property
     def member_of(self):
         """Get groups and directory roles that the user is a direct member of."""
@@ -479,6 +600,21 @@ class User(DirectoryObject):
                                    DeltaCollection(self.context, OAuth2PermissionGrant,
                                                    ResourcePath("oauth2PermissionGrants", self.resource_path)))
 
+    @property
+    def owned_devices(self):
+        """Devices that are owned by the user. Read-only. Nullable. Supports $expand and $filter
+        (/$count eq 0, /$count ne 0, /$count eq 1, /$count ne 1). """
+        return self.properties.get('ownedDevices',
+                                   DirectoryObjectCollection(self.context,
+                                                             ResourcePath("ownedDevices", self.resource_path)))
+
+    @property
+    def owned_objects(self):
+        """Directory objects that are owned by the user. Read-only. Nullable. Supports $expand. """
+        return self.properties.get('ownedObjects',
+                                   DirectoryObjectCollection(self.context,
+                                                             ResourcePath("ownedObjects", self.resource_path)))
+
     @property
     def transitive_member_of(self):
         """Get groups, directory roles that the user is a member of. This API request is transitive, and will also
@@ -491,8 +627,7 @@ class User(DirectoryObject):
     def mail_folders(self):
         """Get the mail folder collection under the root folder of the signed-in user. """
         return self.properties.get('mailFolders',
-                                   DeltaCollection(self.context, MailFolder,
-                                                   ResourcePath("mailFolders", self.resource_path)))
+                                   MailFolderCollection(self.context, ResourcePath("mailFolders", self.resource_path)))
 
     @property
     def outlook(self):
@@ -550,6 +685,13 @@ class User(DirectoryObject):
         return self.properties.get('presence',
                                    Presence(self.context, ResourcePath("presence", self.resource_path)))
 
+    @property
+    def registered_devices(self):
+        """Get the devices that are retistered for the user from the registeredDevices navigation property."""
+        return self.properties.get('registeredDevices',
+                                   DirectoryObjectCollection(self.context,
+                                                             ResourcePath("registeredDevices", self.resource_path)))
+
     @property
     def teamwork(self):
         """A container for the range of Microsoft Teams functionalities that are available per user in the tenant."""
@@ -565,11 +707,13 @@ class User(DirectoryObject):
     def get_property(self, name, default_value=None):
         if default_value is None:
             property_mapping = {
+                "appRoleAssignments": self.app_role_assignments,
                 "businessPhones": self.business_phones,
                 "calendarGroups": self.calendar_groups,
                 "contactFolders": self.contact_folders,
                 "followedSites": self.followed_sites,
                 "licenseDetails": self.license_details,
+                "managedDevices": self.managed_devices,
                 "memberOf": self.member_of,
                 "transitiveMemberOf": self.transitive_member_of,
                 "joinedTeams": self.joined_teams,
@@ -578,7 +722,10 @@ class User(DirectoryObject):
                 "mailboxSettings": self.mailbox_settings,
                 "directReports": self.direct_reports,
                 "onlineMeetings": self.online_meetings,
-                "oauth2PermissionGrants": self.oauth2_permission_grants
+                "oauth2PermissionGrants": self.oauth2_permission_grants,
+                "ownedDevices": self.owned_devices,
+                "ownedObjects": self.owned_objects,
+                "registeredDevices": self.registered_devices
             }
             default_value = property_mapping.get(name, None)
         return super(User, self).get_property(name, default_value)
