@@ -1,5 +1,7 @@
 from office365.entity import Entity
+from office365.entity_collection import EntityCollection
 from office365.intune.audit.event import AuditEventCollection
+from office365.intune.devices.managed import ManagedDevice
 from office365.runtime.paths.resource_path import ResourcePath
 
 
@@ -16,10 +18,18 @@ class DeviceManagement(Entity):
         return self.properties.get("auditEvents", AuditEventCollection(self.context,
                                                                        ResourcePath("auditEvents", self.resource_path)))
 
+    @property
+    def managed_devices(self):
+        """"""
+        return self.properties.get('managedDevices',
+                                   EntityCollection(self.context, ManagedDevice,
+                                                    ResourcePath("managedDevices", self.resource_path)))
+
     def get_property(self, name, default_value=None):
         if default_value is None:
             property_mapping = {
-                "auditEvents": self.audit_events
+                "auditEvents": self.audit_events,
+                "managedDevices": self.managed_devices
             }
             default_value = property_mapping.get(name, None)
         return super(DeviceManagement, self).get_property(name, default_value)
