@@ -1,36 +1,38 @@
 import os
 
+from office365.base_item import BaseItem
+from office365.delta_path import DeltaPath
+from office365.entity_collection import EntityCollection
+from office365.onedrive.analytics.item_activity_stat import ItemActivityStat
+from office365.onedrive.analytics.item_analytics import ItemAnalytics
 from office365.onedrive.driveitems.audio import Audio
+from office365.onedrive.driveitems.conflict_behavior import ConflictBehavior
 from office365.onedrive.driveitems.geo_coordinates import GeoCoordinates
 from office365.onedrive.driveitems.image import Image
 from office365.onedrive.driveitems.item_preview_info import ItemPreviewInfo
 from office365.onedrive.driveitems.photo import Photo
+from office365.onedrive.driveitems.publication_facet import PublicationFacet
 from office365.onedrive.driveitems.special_folder import SpecialFolder
-from office365.onedrive.internal.queries.resumable_file_upload import create_resumable_file_upload_query
-from office365.onedrive.internal.queries.upload_content import create_upload_content_query
-from office365.base_item import BaseItem
-from office365.onedrive.analytics.item_activity_stat import ItemActivityStat
-from office365.onedrive.analytics.item_analytics import ItemAnalytics
-from office365.onedrive.permissions.permission import Permission
-from office365.entity_collection import EntityCollection
-from office365.onedrive.internal.paths.children import ChildrenPath
-from office365.onedrive.driveitems.conflict_behavior import ConflictBehavior
-from office365.onedrive.shares.shared import Shared
-from office365.onedrive.versions.drive_item import DriveItemVersion
+from office365.onedrive.driveitems.thumbnail_set import ThumbnailSet
 from office365.onedrive.files.file import File
 from office365.onedrive.files.system_info import FileSystemInfo
 from office365.onedrive.folders.folder import Folder
-from office365.onedrive.listitems.list_item import ListItem
-from office365.onedrive.driveitems.publication_facet import PublicationFacet
-from office365.onedrive.driveitems.thumbnail_set import ThumbnailSet
-from office365.onedrive.workbooks.workbook import Workbook
+from office365.onedrive.internal.paths.children import ChildrenPath
 from office365.onedrive.internal.paths.url import UrlPath
+from office365.onedrive.internal.queries.resumable_file_upload import create_resumable_file_upload_query
+from office365.onedrive.listitems.item_reference import ItemReference
+from office365.onedrive.listitems.list_item import ListItem
+from office365.onedrive.permissions.collection import PermissionCollection
+from office365.onedrive.permissions.permission import Permission
+from office365.onedrive.shares.shared import Shared
+from office365.onedrive.versions.drive_item import DriveItemVersion
+from office365.onedrive.workbooks.workbook import Workbook
 from office365.runtime.client_result import ClientResult
 from office365.runtime.http.http_method import HttpMethod
+from office365.runtime.paths.resource_path import ResourcePath
 from office365.runtime.queries.create_entity import CreateEntityQuery
 from office365.runtime.queries.function import FunctionQuery
 from office365.runtime.queries.service_operation import ServiceOperationQuery
-from office365.runtime.paths.resource_path import ResourcePath
 from office365.runtime.queries.upload_session import UploadSessionQuery
 from office365.subscriptions.subscription import Subscription
 
@@ -51,11 +53,12 @@ class DriveItem(BaseItem):
         """
         Creates a PowerPoint file
 
-        :param str name:
+        :param str name: File name
         """
         return self.upload(name, None)
 
-    def create_link(self, link_type, scope="", expiration_datetime=None, password=None, message=None):
+    def create_link(self, link_type, scope="", expiration_datetime=None, password=None, message=None,
+                    retain_inherited_permissions=None):
         """
         The createLink action will create a new sharing link if the specified link type doesn't already exist
         for the calling application. If a sharing link of the specified type already exists for the app,
@@ -68,13 +71,17 @@ class DriveItem(BaseItem):
         :param str password: The password of the sharing link that is set by the creator. Optional
             and OneDrive Personal only.
         :param str message:
+        :param bool retain_inherited_permissions: Optional. If true (default), any existing inherited permissions
+            are retained on the shared item when sharing this item for the first time.
+            If false, all existing permissions are removed when sharing for the first time.
         """
         payload = {
             "type": link_type,
             "scope": scope,
             "message": message,
             "expirationDateTime": expiration_datetime,
-            "password": password
+            "password": password,
+            "retainInheritedPermissions": retain_inherited_permissions
         }
         return_type = Permission(self.context)
         self.permissions.add_child(return_type)
@@ -158,60 +165,97 @@ class DriveItem(BaseItem):
 
     def upload(self, name, content):
         """The simple upload API allows you to provide the contents of a new file or update the contents of an
-        existing file in a single API call. This method only supports files up to 4MB in size.
+        existing file in a single API call.
+
+        Note: This method only supports files up to 4MB in size.
 
         :param name: The contents of the request body should be the binary stream of the file to be uploaded.
         :type name: str
         :param content: The contents of the request body should be the binary stream of the file to be uploaded.
         :type content: str or bytes or None
-        :rtype: DriveItem
         """
-        qry = create_upload_content_query(self, name, content)
+        return_type = DriveItem(self.context, UrlPath(name, self.resource_path))
+        qry = ServiceOperationQuery(return_type, "content", None, content, None, return_type)
         self.children.add_child(qry.return_type)
-        self.context.add_query(qry)
-        return qry.return_type
 
-    def get_content(self):
-        """Download the contents of the primary stream (file) of a DriveItem. Only driveItems with the file property
-        can be downloaded. """
-        from office365.onedrive.internal.queries.download_content import create_download_content_query
-        qry = create_download_content_query(self)
+        def _modify_query(request):
+            """
+            :type request: office365.runtime.http.request_options.RequestOptions
+            """
+            request.method = HttpMethod.Put
+
+        self.context.add_query(qry).before_query_execute(_modify_query)
+        return return_type
+
+    def upload_file(self, path_or_file):
+        """Uploads a file
+
+        :param str or typing.IO path_or_file:
+        """
+        if hasattr(path_or_file, 'read'):
+            content = path_or_file.read()
+            name = os.path.basename(path_or_file.name)
+            return self.upload(name, content)
+        else:
+            with open(path_or_file, 'rb') as f:
+                content = f.read()
+            name = os.path.basename(path_or_file)
+            return self.upload(name, content)
+
+    def get_content(self, format_name=None):
+        """
+        Download the contents of the primary stream (file) of a DriveItem. O
+        nly driveItems with the file property can be downloaded.
+
+        :type format_name: str or None
+        """
+        return_type = ClientResult(self.context)
+        action_name = "content"
+        if format_name is not None:
+            action_name = action_name + r"?format={0}".format(format_name)
+        qry = FunctionQuery(self, action_name, None, return_type)
         self.context.add_query(qry)
-        return qry.return_type
+        return return_type
 
     def download(self, file_object):
         """
+        Download the contents of the primary stream (file) of a DriveItem. Only driveItems with the file property
+        can be downloaded
+
         :type file_object: typing.IO
         """
-        result = self.get_content()
 
-        def _content_downloaded(resp):
+        def _save_content(return_type):
             """
-            :type resp: requests.Response
+            :type return_type: ClientResult
             """
-            resp.raise_for_status()
-            file_object.write(result.value)
+            file_object.write(return_type.value)
 
-        self.context.after_execute(_content_downloaded)
+        self.get_content().after_execute(_save_content)
         return self
 
     def download_session(self, file_object, chunk_downloaded=None, chunk_size=1024 * 1024):
         """
-        :type file_object: typing.IO
-        :type chunk_downloaded: (int)->None or None
-        :type chunk_size: int
-        """
-        from office365.onedrive.internal.queries.download_content import create_download_session_content_query
-        qry = create_download_session_content_query(self)
+        By default, file gets downloaded immediately.
+        For a large files reading the whole content of a file at once into memory should be avoided.
 
-        def _construct_download_request(request):
+        This method allows to stream content into the file
+
+        :type file_object: typing.IO
+        :param (int)->None or None chunk_downloaded: A callback
+        :param int chunk_size: The number of bytes it should read into memory.
+        """
+
+        def _construct_request(request):
             """
             :type request: office365.runtime.http.request_options.RequestOptions
             """
             request.stream = True
-            request.method = HttpMethod.Get
 
-        def _process_download_response(response):
+        def _process_response(response):
+            """
+            :type response: requests.Response
+            """
             bytes_read = 0
             for chunk in response.iter_content(chunk_size=chunk_size):
                 bytes_read += len(chunk)
@@ -219,22 +263,22 @@ class DriveItem(BaseItem):
                     chunk_downloaded(bytes_read)
                 file_object.write(chunk)
 
-        self.context.before_execute(_construct_download_request)
-        self.context.after_execute(_process_download_response)
-        self.context.add_query(qry)
+        self.get_content().before_execute(_construct_request)
+        self.context.after_execute(_process_response)
         return self
 
-    def create_folder(self, name):
+    def create_folder(self, name, conflict_behavior=ConflictBehavior.Rename):
         """Create a new folder or DriveItem in a Drive with a specified parent item or path.
 
         :param str name: Folder name
+        :param str conflict_behavior: query parameter to customize the behavior when a conflict occurs.
         """
         return_type = DriveItem(self.context)
         self.children.add_child(return_type)
         payload = {
             "name": name,
             "folder": {},
-            "@microsoft.graph.conflictBehavior": ConflictBehavior.Rename
+            "@microsoft.graph.conflictBehavior": conflict_behavior
         }
         qry = CreateEntityQuery(self.children, payload, return_type)
         self.context.add_query(qry)
@@ -245,60 +289,102 @@ class DriveItem(BaseItem):
 
         :param format_name: Specify the format the item's content should be downloaded as.
         :type format_name: str
-        :rtype: ClientResult
         """
-        from office365.onedrive.internal.queries.download_content import create_download_content_query
-        qry = create_download_content_query(self, format_name)
-        self.context.add_query(qry)
-        return qry.return_type
+        return self.get_content(format_name)
 
-    def copy(self, name, parent_reference=None):
+    def copy(self, name=None, parent=None, conflict_behavior=ConflictBehavior.Fail):
         """Asynchronously creates a copy of an driveItem (including any children), under a new parent item or with a
         new name.
 
-        :type name: str
-        :type parent_reference: office365.onedrive.listitems.item_reference.ItemReference or None
-        """
-        result = ClientResult(self.context)
-        qry = ServiceOperationQuery(self,
-                                    "copy",
-                                    None,
-                                    {
-                                        "name": name,
-                                        "parentReference": parent_reference
-                                    },
-                                    None,
-                                    result
-                                    )
-        self.context.add_query(qry)
-        return result
+        :param str or None name: The new name for the copy. If this isn't provided, the same name will be used as the
+             original.
+        :param office365.onedrive.listitems.item_reference.ItemReference or DriveItem or None parent:  Reference to the
+             parent item the copy will be created in.
+        :param str conflict_behavior: query parameter to customize the behavior when a conflict occurs.
 
-    def move(self, name, parent_reference=None):
+        Returns location for details about how to monitor the progress of the copy, upon accepting the request.
+        """
+        return_type = ClientResult(self.context, str())
+
+        def _create_request(request):
+            """
+            :type request: office365.runtime.http.request_options.RequestOptions
+            """
+            request.url += "?@microsoft.graph.conflictBehavior={0}".format(conflict_behavior)
+
+        def _process_response(resp):
+            """
+            :type resp: requests.Response
+            """
+            resp.raise_for_status()
+            location = resp.headers.get("Location", None)
+            if location is None:
+                return
+            return_type.set_property("__value", location)
+
+        def _create_and_add_query(parent_reference):
+            """
+            :param office365.onedrive.listitems.item_reference.ItemReference or None parent_reference:  Reference to the
+             parent item the copy will be created in.
+            """
+            payload = {
+                "name": name,
+                "parentReference": parent_reference
+            }
+            self.context.before_execute(_create_request)
+            self.context.after_execute(_process_response)
+            qry = ServiceOperationQuery(self, "copy", None, payload, None, return_type)
+            self.context.add_query(qry)
+
+        if isinstance(parent, DriveItem):
+            def _drive_item_loaded():
+                parent_reference = ItemReference(drive_id=parent.parent_reference.driveId, _id=parent.id)
+                _create_and_add_query(parent_reference)
+
+            parent.ensure_property("parentReference", _drive_item_loaded)
+        else:
+            _create_and_add_query(parent)
+        return return_type
+
+    def move(self, name=None, parent=None):
         """To move a DriveItem to a new parent item, your app requests to update the parentReference of the DriveItem
         to move.
 
-        :type name: str
-        :type parent_reference: office365.onedrive.listitems.item_reference.ItemReference
+        :param str name: The new name for the move. If this isn't provided, the same name will be used as the
+             original.
+        :param ItemReference or DriveItem or None parent: Reference to the
+             parent item the move will be created in.
         """
 
-        result = ClientResult(self.context)
-        qry = ServiceOperationQuery(self,
-                                    "move",
-                                    None,
-                                    {
-                                        "name": name,
-                                        "parentReference": parent_reference
-                                    },
-                                    None,
-                                    result
-                                    )
-        self.context.add_query(qry)
+        return_type = ClientResult(self.context, str())
 
-        def _construct_request(request):
-            request.method = HttpMethod.Patch
+        def _create_and_add_query(parent_reference):
+            payload = {
+                "name": name,
+                "parentReference": parent_reference
+            }
 
-        self.context.before_execute(_construct_request)
-        return result
+            def _construct_request(request):
+                request.method = HttpMethod.Patch
+
+            self.context.before_execute(_construct_request)
+            qry = ServiceOperationQuery(self, "move", None, payload, None, return_type)
+            self.context.add_query(qry)
+
+        if isinstance(parent, DriveItem):
+            def _drive_item_loaded():
+                _create_and_add_query(ItemReference(_id=parent.id))
+
+            parent.ensure_property("parentReference", _drive_item_loaded)
+        else:
+            _create_and_add_query(parent)
+        return return_type
+
+    def rename(self, new_name):
+        """Rename a DriveItem
+        :param str new_name: The new name for the rename.
+        """
+        return self.move(name=new_name)
 
     def search(self, query_text):
         """Search the hierarchy of items for items matching a query. You can search within a folder hierarchy,
@@ -348,23 +434,30 @@ class DriveItem(BaseItem):
         :param datetime.datetime end_dt: The end time over which to aggregate activities.
         :param str interval: The aggregation interval.
         """
-        params = {
-            "startDateTime": start_dt.strftime('%m-%d-%Y') if start_dt else None,
-            "endDateTime": end_dt.strftime('%m-%d-%Y') if end_dt else None,
-            "interval": interval
-        }
         return_type = EntityCollection(self.context, ItemActivityStat, self.resource_path)
-        qry = FunctionQuery(self, "getActivitiesByInterval", params, return_type)
+
+        def _create_query():
+            params = {
+                "startDateTime": start_dt.strftime('%m-%d-%Y') if start_dt else None,
+                "endDateTime": end_dt.strftime('%m-%d-%Y') if end_dt else None,
+                "interval": interval
+            }
+            return FunctionQuery(self, "getActivitiesByInterval", params, return_type)
+
+        qry = _create_query()
         self.context.add_query(qry)
+
         return return_type
 
-    def restore(self, parent_reference, name):
+    def restore(self, parent_reference=None, name=None):
         """
         Restore a driveItem that has been deleted and is currently in the recycle bin.
         NOTE: This functionality is currently only available for OneDrive Personal.
 
-        :type name: str
-        :type parent_reference: office365.onedrive.listitems.item_reference.ItemReference or None
+        :param str name: Optional. The new name for the restored item. If this isn't provided,
+             the same name will be used as the original.
+        :param ItemReference or None parent_reference: Optional. Reference to the parent item the deleted item will
+             be restored to.
         """
         payload = {
             "name": name,
@@ -390,10 +483,10 @@ class DriveItem(BaseItem):
             "page": page,
             "zoom": zoom
         }
-        result = ClientResult(self.context, ItemPreviewInfo())
-        qry = ServiceOperationQuery(self, "preview", None, payload, None, result)
+        return_type = ClientResult(self.context, ItemPreviewInfo())
+        qry = ServiceOperationQuery(self, "preview", None, payload, None, return_type)
         self.context.add_query(qry)
-        return result
+        return return_type
 
     def validate_permission(self, challenge_token=None, password=None):
         """
@@ -480,11 +573,9 @@ class DriveItem(BaseItem):
     def children(self):
         """Collection containing Item objects for the immediate children of Item. Only items representing folders
         have children.
-
-        :rtype: EntityCollection
         """
-        return self.get_property('children',
-                                 EntityCollection(self.context, DriveItem, ChildrenPath(self.resource_path)))
+        return self.properties.get('children',
+                                   EntityCollection(self.context, DriveItem, ChildrenPath(self.resource_path)))
 
     @property
     def listItem(self):
@@ -500,8 +591,7 @@ class DriveItem(BaseItem):
     def permissions(self):
         """The set of permissions for the item. Read-only. Nullable."""
         return self.properties.get('permissions',
-                                   EntityCollection(self.context, Permission,
-                                                    ResourcePath("permissions", self.resource_path)))
+                                   PermissionCollection(self.context, ResourcePath("permissions", self.resource_path)))
 
     @property
     def publication(self):
@@ -540,7 +630,7 @@ class DriveItem(BaseItem):
     def delta(self):
         """This method allows your app to track changes to a drive item and its children over time."""
         return self.properties.get('delta',
-                                   EntityCollection(self.context, DriveItem, ResourcePath("delta", self.resource_path)))
+                                   EntityCollection(self.context, DriveItem, DeltaPath(self.resource_path)))
 
     @property
     def subscriptions(self):
