@@ -9,7 +9,10 @@ class DriveRecipient(ClientValue):
 
     def __init__(self, alias=None, email=None, object_id=None):
         """
-
+        :param str alias: The alias of the domain object, for cases where an email address is unavailable
+            (e.g. security groups).
+        :param str email: The email address for the recipient, if the recipient has an associated email address.
+        :param str object_id: The unique identifier for the recipient in the directory.
         """
         super(DriveRecipient, self).__init__()
         self.alias = alias
