@@ -4,6 +4,10 @@ from office365.runtime.paths.resource_path import ResourcePath
 
 
 def _url_to_shared_token(url):
+    """
+    Converts url into shared token
+    :param str url:
+    """
     value = base64.b64encode(url.encode("ascii")).decode("ascii")
     if value.endswith("="):
         value = value[:-1]
@@ -13,5 +17,6 @@ def _url_to_shared_token(url):
 class SharedPath(ResourcePath):
     """Shared token path"""
 
-    def __init__(self, url, parent=None):
-        super(SharedPath, self).__init__(_url_to_shared_token(url), parent)
+    @property
+    def segment(self):
+        return _url_to_shared_token(self.key)
