@@ -1,7 +1,6 @@
 from office365.entity_collection import EntityCollection
 from office365.onedrive.analytics.item_activity_stat import ItemActivityStat
-from office365.runtime.http.http_method import HttpMethod
-from office365.runtime.queries.service_operation import ServiceOperationQuery
+from office365.runtime.queries.function import FunctionQuery
 
 
 def build_get_activities_by_interval_query(binding_type, start_dt=None, end_dt=None, interval=None):
@@ -17,9 +16,5 @@ def build_get_activities_by_interval_query(binding_type, start_dt=None, end_dt=N
         "interval": interval
     }
     return_type = EntityCollection(binding_type.context, ItemActivityStat, binding_type.resource_path)
-    qry = ServiceOperationQuery(binding_type, "getActivitiesByInterval", params, None, None, return_type)
-
-    def _construct_request(request):
-        request.method = HttpMethod.Get
-    binding_type.context.before_execute(_construct_request)
+    qry = FunctionQuery(binding_type, "getActivitiesByInterval", params, return_type)
     return qry
