@@ -1,7 +1,6 @@
 from office365.onenote.entity_base_model import OnenoteEntityBaseModel
 from office365.runtime.client_result import ClientResult
-from office365.runtime.http.http_method import HttpMethod
-from office365.runtime.queries.service_operation_query import ServiceOperationQuery
+from office365.runtime.queries.function import FunctionQuery
 
 
 class OnenoteResource(OnenoteEntityBaseModel):
@@ -9,17 +8,10 @@ class OnenoteResource(OnenoteEntityBaseModel):
 
     def get_content(self):
         """Retrieve the binary data of a file or image resource object."""
-        result = ClientResult(self.context)
-        qry = ServiceOperationQuery(self, "content", None, None, None, result)
-
-        def _construct_query(request):
-            """
-            :type request: office365.runtime.http.request_options.RequestOptions
-            """
-            request.method = HttpMethod.Get
-        self.context.before_execute(_construct_query)
+        return_type = ClientResult(self.context)
+        qry = FunctionQuery(self, "content", None, return_type)
         self.context.add_query(qry)
-        return result
+        return return_type
 
     @property
     def content_url(self):
