@@ -12,6 +12,8 @@ class CalendarGroup(Entity):
     @property
     def name(self):
         """
+        The group name.
+
         :rtype: str
         """
         return self.properties.get("name", None)
@@ -19,6 +21,8 @@ class CalendarGroup(Entity):
     @property
     def class_id(self):
         """
+        The class identifier
+
         :rtype: str
         """
         return self.properties.get("classId", None)
