@@ -6,6 +6,13 @@ class Place(Entity):
     """Represents basic location attributes such as name, physical address, and geographic coordinates.
     This is the base type for richer location types such as room and roomList."""
 
+    @property
+    def display_name(self):
+        """The name associated with the place.
+        :rtype: str or None
+        """
+        return self.properties.get("displayName", None)
+
     @property
     def address(self):
         """The street address of the place."""
