@@ -4,8 +4,8 @@ class ClientCredential(object):
         """
         Client credentials
 
-        :type client_secret: str
-        :type client_id: str
+        :param str client_secret:
+        :param str client_id:
         """
         self.clientId = client_id
         self.clientSecret = client_secret
