@@ -30,6 +30,9 @@ class ClientObject(object):
         self._resource_path = resource_path
 
     def clear(self):
+        """
+        Resets client object's state
+        """
         self._properties = {k: v for k, v in self._properties.items() if k not in self._ser_property_names}
         self._ser_property_names = []
         self._query_options = QueryOptions()
@@ -50,6 +53,13 @@ class ClientObject(object):
                             failure_callback=None):
         """
         Executes the current set of data retrieval queries and method invocations and retries it if needed.
+
+        :param int max_retry: Number of times to retry the request
+        :param int timeout_secs: Seconds to wait before retrying the request.
+        :param (office365.runtime.client_object.ClientObject)-> None success_callback: A callback to call
+            if the request executes successfully.
+        :param (int, requests.exceptions.RequestException)-> None failure_callback: A callback to call if the request
+            fails to execute
         """
         self.context.execute_query_retry(max_retry=max_retry,
                                          timeout_secs=timeout_secs,
@@ -57,11 +67,17 @@ class ClientObject(object):
                                          failure_callback=failure_callback)
         return self
 
+    def after_execute(self, action, *args, **kwargs):
+        """
+        Attach an event handler to client object which gets triggered after query is submitted to server
+        """
+        self._context.after_query_execute(action, self, *args, **kwargs)
+        return self
+
     def get(self):
         """
-        :type self: T
-
         Retrieves a client object from the server
+        :type self: T
         """
         self.context.load(self)
         return self
@@ -101,10 +117,13 @@ class ClientObject(object):
         self._parent_collection.remove_child(self)
         return self
 
-    def track_changes(self, name, value):
+    def _persist_changes(self, name):
+        """
+        Marks a property as a serializable
+        :param str name: A property name
+        """
         if name not in self._ser_property_names:
             self._ser_property_names.append(name)
-            self._properties[name] = value
         return self
 
     def get_property(self, name, default_value=None):
@@ -158,17 +177,19 @@ class ClientObject(object):
 
     def ensure_properties(self, names, action, *args, **kwargs):
         """
-        Ensure if list of properties are loaded
+        Ensure if list of properties are retrieved from the server
 
         :type action: (any) -> None
         :type names: str or list[str]
         """
 
         names_to_include = [n for n in names if not self.is_property_available(n)]
+        if self.property_ref_name is not None and self.property_ref_name not in names_to_include:
+            names_to_include.append(self.property_ref_name)
         if len(names_to_include) > 0:
             from office365.runtime.queries.read_entity import ReadEntityQuery
             qry = ReadEntityQuery(self, names_to_include)
-            self.context.add_query(qry).after_query_execute(qry, action, *args, **kwargs)
+            self.context.add_query(qry).after_query_execute(action, *args, **kwargs)
         else:
             action(*args, **kwargs)
         return self
@@ -180,6 +201,14 @@ class ClientObject(object):
             self._entity_type_name = type(self).__name__
         return self._entity_type_name
 
+    @property
+    def property_ref_name(self):
+        """Returns property reference name
+
+        :rtype: str
+        """
+        return None
+
     @property
     def resource_url(self):
         """
@@ -209,6 +238,7 @@ class ClientObject(object):
 
     @property
     def parent_collection(self):
+        """Parent collection"""
         return self._parent_collection
 
     def to_json(self, json_format=None):
