@@ -89,9 +89,30 @@ class ClientRuntimeContext(object):
         if callable(before_loaded):
             self.before_execute(before_loaded)
         if callable(after_loaded):
-            def _action():
-                after_loaded(client_object)
-            self.after_query_execute(qry, _action)
+            self.after_query_execute(after_loaded, client_object)
+        return self
+
+    def before_query_execute(self, action, once=True, *args, **kwargs):
+        """
+        Attach an event handler which is triggered before query is submitted to server
+
+        :type action: (office365.runtime.http.request_options.RequestOptions, *args, **kwargs) -> None
+        :param bool once: Flag which determines whether action is executed once or multiple times
+        """
+
+        query = self._queries[-1]
+
+        def _prepare_request(request):
+            """
+            :type request: office365.runtime.http.request_options.RequestOptions
+            """
+
+            if self.current_query.id == query.id:
+                if once:
+                    self.pending_request().beforeExecute -= _prepare_request
+                action(request, *args, **kwargs)
+
+        self.pending_request().beforeExecute += _prepare_request
         return self
 
     def before_execute(self, action, once=True, *args, **kwargs):
@@ -110,13 +131,13 @@ class ClientRuntimeContext(object):
         self.pending_request().beforeExecute += _process_request
         return self
 
-    def after_query_execute(self, query, action, *args, **kwargs):
+    def after_query_execute(self, action, *args, **kwargs):
         """
         Attach an event handler which is triggered after query is submitted to server
 
-        :type query: office365.runtime.queries.client_query.ClientQuery
         :type action: (Response, *args, **kwargs) -> None
         """
+        query = self._queries[-1]
 
         def _process_response(resp):
             """
@@ -128,6 +149,11 @@ class ClientRuntimeContext(object):
                 action(*args, **kwargs)
 
         self.pending_request().afterExecute += _process_response
+
+        execute_first = kwargs.pop("execute_first", False)
+        if execute_first and len(self._queries) > 1:
+            self._queries.insert(0, self._queries.pop())
+
         return self
 
     def after_execute(self, action, once=True, *args, **kwargs):
