@@ -3,12 +3,12 @@ from office365.runtime.paths.resource_path import ResourcePath
 
 class EntityPath(ResourcePath):
 
-    def __init__(self, name=None, parent=None, collection=None):
+    def __init__(self, key=None, parent=None, collection=None):
         """
-        :param str or None name:
+        :param str or None key: Entity key
         :param ResourcePath or None collection:
         """
-        super(EntityPath, self).__init__(name, parent)
+        super(EntityPath, self).__init__(key, parent)
         self._collection = collection
 
     @property
@@ -16,20 +16,20 @@ class EntityPath(ResourcePath):
         return self._collection
 
     @property
-    def segments(self):
-        return [self.delimiter, str(self.name or '<id>')]
+    def segment(self):
+        return str(self.key or '<key>')
 
-    def normalize(self, name, inplace=False):
+    def patch(self, key, inplace=False):
         """
-        Normalizes entity path
+        Patches path
 
-        :type name: str or None
+        :type key: str or None
         :type inplace: bool
         """
         if inplace:
-            self._name = name
+            self._key = key
             self._parent = self.collection
             self.__class__ = ResourcePath
             return self
         else:
-            return ResourcePath(name, self.collection)
+            return ResourcePath(key, self.collection)
