@@ -1,5 +1,5 @@
+from office365.runtime.odata.url_builder import ODataUrlBuilder
 from office365.runtime.paths.resource_path import ResourcePath
-from office365.runtime.odata.path_builder import ODataPathBuilder
 
 
 class ServiceOperationPath(ResourcePath):
@@ -15,5 +15,9 @@ class ServiceOperationPath(ResourcePath):
         self._parameters = parameters
 
     @property
-    def segments(self):
-        return [self.delimiter, ODataPathBuilder.build(self._name, self._parameters)]
+    def segment(self):
+        return ODataUrlBuilder.build_segment(self)
+
+    @property
+    def parameters(self):
+        return self._parameters
