@@ -1,9 +1,10 @@
 import copy
 
 from office365.runtime.auth.authentication_context import AuthenticationContext
+from office365.runtime.auth.client_credential import ClientCredential
 from office365.runtime.auth.user_credential import UserCredential
 from office365.runtime.client_runtime_context import ClientRuntimeContext
-from office365.runtime.compat import urlparse, is_absolute_url, get_absolute_url
+from office365.runtime.compat import urlparse, get_absolute_url
 from office365.runtime.http.http_method import HttpMethod
 from office365.runtime.http.request_options import RequestOptions
 from office365.runtime.odata.request import ODataRequest
@@ -22,7 +23,7 @@ from office365.sharepoint.webs.web import Web
 
 
 class ClientContext(ClientRuntimeContext):
-    """SharePoint client context"""
+    """SharePoint client context (used for SharePoint v1 API)"""
 
     def __init__(self, base_url, auth_context=None):
         """
@@ -40,45 +41,24 @@ class ClientContext(ClientRuntimeContext):
         self._ctx_web_info = None
         self._pending_request = None
 
-    def create_safe_url(self, orig_url, relative=True):
-        """
-        Creates a safe url
-
-        :type orig_url: str
-        :type relative: bool
-        """
-        if is_absolute_url(orig_url) and not relative:
-            return orig_url
-
-        site_path = urlparse(self.base_url).path
-        root_site_url = self.base_url.replace(site_path, "")
-        url = orig_url if orig_url.startswith(site_path) else "/".join([site_path, orig_url])
-        return url if relative else "".join([root_site_url, url])
-
     @staticmethod
     def from_url(full_url):
         """
-        Constructs ClientContext from absolute Url
+        Constructs a client from absolute resource url
 
-        :param str full_url: Full Url to a resource
-        :return: ClientContext
+        :param str full_url: Absolute Url to a resource
         """
         root_site_url = get_absolute_url(full_url)
         ctx = ClientContext(root_site_url)
-        result = Web.get_web_url_from_page_url(ctx, full_url)
 
-        def _init_context_for_web(resp):
-            """
-            :type resp: requests.Response
-            """
-            resp.raise_for_status()
-            ctx._auth_context.url = result.value
-
-        ctx.after_execute(_init_context_for_web)
+        def _init_context(return_type):
+            ctx._auth_context.url = return_type.value
+        Web.get_web_url_from_page_url(ctx, full_url).after_execute(_init_context)
         return ctx
 
     def with_client_certificate(self, tenant, client_id, thumbprint, cert_path=None, private_key=None, scopes=None):
-        """Creates authenticated SharePoint context via certificate credentials
+        """
+        Creates authenticated SharePoint context via certificate credentials
 
         :param str tenant: Tenant name
         :param str or None cert_path: Path to A PEM encoded certificate private key.
@@ -92,21 +72,49 @@ class ClientContext(ClientRuntimeContext):
                                                             scopes)
         return self
 
+    def with_interactive(self, tenant, client_id, scopes=None):
+        """
+        Initializes a client to acquire a token interactively i.e. via a local browser.
+
+        Prerequisite: In Azure Portal, configure the Redirect URI of your
+        "Mobile and Desktop application" as ``http://localhost``.
+
+        :param str tenant: Tenant name, for example: contoso.onmicrosoft.com
+        :param str client_id: The OAuth client id of the calling application.
+        :param list[str] or None scopes:  Scopes requested to access a protected API (a resource)
+        """
+        self.authentication_context.with_interactive(tenant, client_id, scopes)
+        return self
+
+    def with_device_flow(self, tenant, client_id, scopes=None):
+        """
+        Initializes a client to acquire a token via device flow auth.
+
+        :param str tenant: Tenant name, for example: contoso.onmicrosoft.com
+        :param str client_id: The OAuth client id of the calling application.
+        :param list[str] or None scopes:  Scopes requested to access a protected API (a resource)
+        """
+        self.authentication_context.with_device_flow(tenant, client_id, scopes)
+        return self
+
     def with_access_token(self, token_func):
         """
-        :type token_func: () -> TokenResponse
+        Initializes a client to acquire a token from a callback
+
+        :param () -> TokenResponse token_func: A callback
         """
         self.authentication_context.with_access_token(token_func)
         return self
 
     def with_user_credentials(self, username, password, allow_ntlm=False, browser_mode=False):
         """
-        Assigns credentials
+        Initializes a client to acquire a token via user credentials.
 
-        :type username: str
-        :type password: str
-        :type allow_ntlm: bool
-        :type browser_mode: bool
+
+        :param str username: Typically, a UPN in the form of an email address
+        :param str password: The password
+        :param bool allow_ntlm: Flag indicates whether NTLM scheme is enabled. Disabled by default
+        :param bool browser_mode:
         """
         self.authentication_context.with_credentials(
             UserCredential(username, password),
@@ -114,9 +122,22 @@ class ClientContext(ClientRuntimeContext):
             browser_mode=browser_mode)
         return self
 
+    def with_client_credentials(self, client_id, client_secret):
+        """
+        Initializes a client to acquire a token via client credentials (SharePoint App-Only)
+
+        SharePoint App-Only is the older, but still very relevant, model of setting up app-principals.
+        This model works for both SharePoint Online and SharePoint 2013/2016/2019 on-premises
+
+        :param str client_id: The OAuth client id of the calling application
+        :param str client_secret: Secret string that the application uses to prove its identity when requesting a token
+        """
+        self.authentication_context.with_credentials(ClientCredential(client_id, client_secret))
+        return self
+
     def with_credentials(self, credentials):
         """
-        Assigns credentials
+        Initializes a client to acquire a token via user or client credentials
 
         :type credentials: UserCredential or ClientCredential
         """
@@ -125,7 +146,7 @@ class ClientContext(ClientRuntimeContext):
 
     def execute_batch(self, items_per_batch=100):
         """
-        Construct and submit a batch request
+        Construct and submit to a server a batch request
 
         :param int items_per_batch: Maximum to be selected for bulk operation
         """
@@ -158,9 +179,13 @@ class ClientContext(ClientRuntimeContext):
         request.set_header('X-RequestDigest', self._ctx_web_info.FormDigestValue)
 
     def _get_context_web_information(self):
-        """Returns an ContextWebInformation object that specifies metadata about the site"""
+        """
+        Returns an ContextWebInformation object that specifies metadata about the site
+        """
         client = ODataRequest(JsonLightFormat())
-        client.beforeExecute += self._authenticate_request
+        for e in self.pending_request().beforeExecute:
+            client.beforeExecute += e
+        client.beforeExecute -= self._build_modification_query
         request = RequestOptions("{0}/contextInfo".format(self.service_root_url()))
         request.method = HttpMethod.Post
         response = client.execute_request_direct(request)
@@ -209,6 +234,10 @@ class ClientContext(ClientRuntimeContext):
         return ctx
 
     def _authenticate_request(self, request):
+        """
+        Authenticate request
+        :type request: office365.runtime.http.request_options.RequestOptions
+        """
         self.authentication_context.authenticate_request(request)
 
     def _build_modification_query(self, request):
@@ -220,7 +249,7 @@ class ClientContext(ClientRuntimeContext):
         if request.method == HttpMethod.Post:
             self._ensure_form_digest(request)
         # set custom SharePoint control headers
-        if isinstance(self.pending_request().default_json_format, JsonLightFormat):
+        if isinstance(self.pending_request().json_format, JsonLightFormat):
             if isinstance(self.current_query, DeleteEntityQuery):
                 request.ensure_header("X-HTTP-Method", "DELETE")
                 request.ensure_header("IF-MATCH", '*')
@@ -284,25 +313,23 @@ class ClientContext(ClientRuntimeContext):
         """
         return_type = Site(self)
         site_url = "{base_url}/sites/{alias}".format(base_url=get_absolute_url(self.base_url), alias=alias)
-        site_result = self.site_pages.communication_site.create(title, site_url)
 
-        def _after_site_create(resp):
+        def _after_site_created(result):
             """
-            :type resp: requests.Response
+            :type result: ClientResult
             """
-            resp.raise_for_status()
-            if site_result.value.SiteStatus == SiteStatus.Error:
+            if result.value.SiteStatus == SiteStatus.Error:
                 raise ValueError("Site creation error")
-            elif site_result.value.SiteStatus == SiteStatus.Ready:
-                return_type.set_property("__siteUrl", site_result.value.SiteUrl)
-        self.after_execute(_after_site_create)
+            elif result.value.SiteStatus == SiteStatus.Ready:
+                return_type.set_property("__siteUrl", result.value.SiteUrl)
+
+        self.site_pages.communication_site.create(title, site_url).after_execute(_after_site_created)
         return return_type
 
     @property
     def context_info(self):
-        """Returns an ContextWebInformation object that specifies metadata about the site
-
-        :rtype: ContextWebInformation
+        """
+        Returns an ContextWebInformation object that specifies metadata about the site
         """
         if self._ctx_web_info is None:
             self._ctx_web_info = ContextWebInformation()
@@ -310,20 +337,26 @@ class ClientContext(ClientRuntimeContext):
 
     @property
     def web(self):
-        """Get Web client object"""
+        """
+        A group of related webpages that is hosted by a server on the World Wide Web or an intranet.
+        Each website has its own entry points, metadata, administration settings, and workflows.
+        """
         if not self._web:
             self._web = Web(self)
         return self._web
 
     @property
     def site(self):
-        """Get Site client object"""
+        """
+        Represents a collection of sites in a Web application, including a top-level website and all its sub sites.
+        """
         if not self._site:
             self._site = Site(self)
         return self._site
 
     @property
     def apps(self):
+        """"""
         from office365.sharepoint.apps.app_collection import AppCollection
         return AppCollection(self, ResourcePath("Apps"))
 
@@ -338,6 +371,12 @@ class ClientContext(ClientRuntimeContext):
         from office365.sharepoint.viva.employee_engagement import EmployeeEngagement
         return EmployeeEngagement(self)
 
+    @property
+    def employee_experience(self):
+        """Alias to EmployeeExperience"""
+        from office365.sharepoint.viva.employee_experience_controller import EmployeeExperienceController
+        return EmployeeExperienceController(self)
+
     @property
     def micro_service_manager(self):
         """Alias to MicroServiceManager"""
@@ -456,6 +495,12 @@ class ClientContext(ClientRuntimeContext):
         from office365.sharepoint.portal.sites.manager import SPSiteManager
         return SPSiteManager(self, ResourcePath("spSiteManager"))
 
+    @property
+    def social_feed_manager(self):
+        """Alias to SocialFeedManager."""
+        from office365.sharepoint.social.feed.manager import SocialFeedManager
+        return SocialFeedManager(self)
+
     @property
     def home_service(self):
         """Alias to SharePointHomeServiceContextBuilder."""
@@ -503,8 +548,20 @@ class ClientContext(ClientRuntimeContext):
         from office365.sharepoint.tenant.settings import TenantSettings
         return TenantSettings.current(self)
 
+    @property
+    def viva_site_manager(self):
+        from office365.sharepoint.viva.site_manager import VivaSiteManager
+        return VivaSiteManager(self)
+
+    @property
+    def workflow_services_manager(self):
+        """Alias to WorkflowServicesManager"""
+        from office365.sharepoint.workflowservices.manager import WorkflowServicesManager
+        return WorkflowServicesManager.current(self)
+
     @property
     def work_items(self):
+        """"""
         from office365.sharepoint.contentcenter.machinelearning.workitems.collection import \
             SPMachineLearningWorkItemCollection
         return SPMachineLearningWorkItemCollection(self, ResourcePath("workitems"))
