@@ -2,6 +2,7 @@ import json
 
 from office365.runtime.client_result import ClientResult
 from office365.runtime.client_value_collection import ClientValueCollection
+from office365.runtime.paths.key import KeyPath
 from office365.runtime.queries.service_operation import ServiceOperationQuery
 from office365.runtime.paths.resource_path import ResourcePath
 from office365.runtime.paths.service_operation import ServiceOperationPath
@@ -58,11 +59,12 @@ class ListItem(SecurableObject):
             between the local and UTC time. Midnight is represented as 00:00:00. A null value indicates no expiry.
             This value is only applicable to tokenized sharing links that are anonymous access links.
         """
-        result = ClientResult(self.context, ShareLinkResponse())
-        payload = ShareLinkRequest(settings=ShareLinkSettings(link_kind=link_kind, expiration=expiration))
-        qry = ServiceOperationQuery(self, "ShareLink", None, payload, "request", result)
+        return_type = ClientResult(self.context, ShareLinkResponse())
+        request = ShareLinkRequest(settings=ShareLinkSettings(link_kind=link_kind, expiration=expiration))
+        payload = {"request": request}
+        qry = ServiceOperationQuery(self, "ShareLink", None, payload, None, return_type)
         self.context.add_query(qry)
-        return result
+        return return_type
 
     def unshare_link(self, link_kind, share_id=None):
         """
@@ -151,10 +153,11 @@ class ListItem(SecurableObject):
         """
         if query is None:
             query = ChangeQuery(item=True)
-        changes = ChangeCollection(self.context)
-        qry = ServiceOperationQuery(self, "getChanges", None, query, "query", changes)
+        return_type = ChangeCollection(self.context)
+        payload = {"query": query}
+        qry = ServiceOperationQuery(self, "getChanges", None, payload, None, return_type)
         self.context.add_query(qry)
-        return changes
+        return return_type
 
     def share(self, user_principal_name,
               share_option=ExternalSharingSiteOption.View,
@@ -265,9 +268,9 @@ class ListItem(SecurableObject):
 
     def set_comments_disabled(self, value):
         """
-        Sets the value of CommentsDisabled (section 3.2.5.87.1.1.8) for the item.
+        Sets the value of CommentsDisabled for the item.
 
-        :type value: bool
+        :param bool value: Indicates whether comments for this item are disabled or not.
         """
         qry = ServiceOperationQuery(self, "SetCommentsDisabled", [value])
         self.context.add_query(qry)
@@ -471,33 +474,43 @@ class ListItem(SecurableObject):
             super(ListItem, self).set_property(name, value, persist_changes)
 
         # fallback: create a new resource path
-        if self._resource_path is None:
+        if self._resource_path is None and self.parent_collection is not None:
             if name == "Id":
-                self._resource_path = ServiceOperationPath(
-                    "getItemById", [value], self._parent_collection.resource_path.parent)
+                self._resource_path = KeyPath(value, self.parent_collection.resource_path)
         return self
 
     def _set_taxonomy_field_value(self, name, value):
+        """
+        Sets taxonomy field value
+
+        :param str name: Taxonomy field name
+        :param TaxonomyFieldValueCollection value: Taxonomy field value
+        """
         tax_field = self.parent_list.fields.get_by_internal_name_or_title(name)
 
         def _tax_field_loaded():
             tax_text_field = self.parent_list.fields.get_by_id(tax_field.properties["TextField"])
 
-            def _tax_text_field_loaded():
+            def _tax_text_field_loaded(return_type):
                 self.set_property(tax_text_field.properties["StaticName"], str(value))
-            tax_text_field.ensure_property("StaticName", _tax_text_field_loaded)
+            tax_text_field.select(["StaticName"]).get().after_execute(_tax_text_field_loaded, execute_first=True)
 
         tax_field.ensure_property("TextField", _tax_field_loaded)
 
-    def ensure_type_name(self, target_list):
+    def ensure_type_name(self, target_list, action=None):
         """
         Determine metadata annotation for ListItem entity
 
         :param office365.sharepoint.lists.list.List target_list: List resource
+        :param () -> None action: Event handler
         """
-
-        def _init_item_type():
-            self._entity_type_name = target_list.properties['ListItemEntityTypeFullName']
-
-        if not self._entity_type_name:
-            target_list.ensure_property("ListItemEntityTypeFullName", _init_item_type)
+        if self._entity_type_name is None:
+            def _list_loaded():
+                self._entity_type_name = target_list.properties['ListItemEntityTypeFullName']
+                if callable(action):
+                    action()
+            target_list.ensure_property("ListItemEntityTypeFullName", _list_loaded)
+        else:
+            if callable(action):
+                action()
+        return self
