@@ -2,6 +2,7 @@ from office365.runtime.client_result import ClientResult
 from office365.runtime.queries.service_operation import ServiceOperationQuery
 from office365.runtime.paths.resource_path import ResourcePath
 from office365.sharepoint.base_entity import BaseEntity
+from office365.sharepoint.logger.logFileInfoCollection import LogFileInfoCollection
 
 
 class LogExport(BaseEntity):
@@ -11,11 +12,18 @@ class LogExport(BaseEntity):
         logs that you can download."""
         super(LogExport, self).__init__(context, ResourcePath("Microsoft.Online.SharePoint.SPLogger.LogExport"))
 
-    def get_files(self, partitionId, logType):
-        pass
+    def get_files(self):
+        return_type = LogFileInfoCollection(self.context)
+        qry = ServiceOperationQuery(self, "GetFiles", None, None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
 
     def get_log_types(self):
-        result = ClientResult(self.context)
-        qry = ServiceOperationQuery(self, "GetLogTypes")
+        return_type = ClientResult(self.context)
+        qry = ServiceOperationQuery(self, "GetLogTypes", None, None, None, return_type)
         self.context.add_query(qry)
-        return result
+        return return_type
+
+    @property
+    def entity_type_name(self):
+        return "Microsoft.Online.SharePoint.SPLogger.LogExport"
