@@ -1,7 +1,6 @@
 from office365.runtime.paths.resource_path import ResourcePath
 from office365.sharepoint.base_entity import BaseEntity
-from office365.sharepoint.base_entity_collection import BaseEntityCollection
-from office365.sharepoint.marketplace.app_metadata import CorporateCatalogAppMetadata
+from office365.sharepoint.marketplace.app_metadata_collection import CorporateCatalogAppMetadataCollection
 
 
 class SiteCollectionCorporateCatalogAccessor(BaseEntity):
@@ -11,8 +10,9 @@ class SiteCollectionCorporateCatalogAccessor(BaseEntity):
     def available_apps(self):
         """Returns the apps available in this corporate catalog."""
         return self.properties.get('AvailableApps',
-                                   BaseEntityCollection(self.context, CorporateCatalogAppMetadata,
-                                                        ResourcePath("AvailableApps", self.resource_path)))
+                                   CorporateCatalogAppMetadataCollection(self.context,
+                                                                         ResourcePath("AvailableApps",
+                                                                                      self.resource_path)))
 
     def get_property(self, name, default_value=None):
         if default_value is None:
