@@ -1,11 +1,14 @@
 from office365.runtime.client_result import ClientResult
+from office365.runtime.client_value_collection import ClientValueCollection
 from office365.runtime.queries.service_operation import ServiceOperationQuery
 from office365.runtime.paths.resource_path import ResourcePath
 from office365.sharepoint.base_entity import BaseEntity
 from office365.sharepoint.permissions.base_permissions import BasePermissions
 from office365.sharepoint.permissions.roles.assignments.assignment import RoleAssignment
 from office365.sharepoint.permissions.roles.assignments.collection import RoleAssignmentCollection
-from office365.sharepoint.principal.user import User
+from office365.sharepoint.principal.users.user import User
+from office365.sharepoint.sharing.document_manager import DocumentSharingManager
+from office365.sharepoint.sharing.user_sharing_result import UserSharingResult
 
 
 class SecurableObject(BaseEntity):
@@ -28,14 +31,13 @@ class SecurableObject(BaseEntity):
         return return_type
 
     def add_role_assignment(self, principal, role_def):
-        """Adds a role assignment to securable resource.<81>
+        """Adds a role assignment to securable resource.
 
-        :param office365.sharepoint.permissions.role_definition.RoleDefinition role_def: Specifies the role definition
-        of the role assignment.
+        :param office365.sharepoint.permissions.roles.definitions.definition.RoleDefinition role_def: Specifies the
+            role definition of the role assignment.
         :param office365.sharepoint.principal.principal.Principal principal: Specifies the user or group of the
-        role assignment.
+            role assignment.
         """
-
         def _principal_loaded():
             role_def.ensure_property("Id", _role_def_loaded)
 
@@ -97,24 +99,28 @@ class SecurableObject(BaseEntity):
         self.context.add_query(qry)
         return self
 
-    def get_user_effective_permissions(self, user_or_name):
+    def get_user_effective_permissions(self, user):
         """
         Returns the user permissions for this list.
 
-        :param str or User user_or_name: Specifies the user login name or User object.
+        :param str or User user: Specifies the user login name or User object.
         """
-        result = ClientResult(self.context, BasePermissions())
+        return_type = ClientResult(self.context, BasePermissions())
 
-        if isinstance(user_or_name, User):
-            def _user_loaded():
-                next_qry = ServiceOperationQuery(self, "GetUserEffectivePermissions", [user_or_name.login_name],
-                                                 None, None, result)
-                self.context.add_query(next_qry)
-            user_or_name.ensure_property("LoginName", _user_loaded)
-        else:
-            qry = ServiceOperationQuery(self, "GetUserEffectivePermissions", [user_or_name], None, None, result)
+        def _create_and_add_query(login_name):
+            """
+            :param str login_name:
+            """
+            qry = ServiceOperationQuery(self, "GetUserEffectivePermissions", [login_name], None, None, return_type)
             self.context.add_query(qry)
-        return result
+
+        if isinstance(user, User):
+            def _user_loaded():
+                _create_and_add_query(user.login_name)
+            user.ensure_property("LoginName", _user_loaded)
+        else:
+            _create_and_add_query(user)
+        return return_type
 
     @property
     def has_unique_role_assignments(self):
