@@ -3,4 +3,7 @@ from office365.runtime.client_value import ClientValue
 
 class SiteHealthResult(ClientValue):
     """Specifies the result of running a site collection health rule."""
-    pass
+
+    @property
+    def entity_type_name(self):
+        return "SP.SiteHealth.SiteHealthResult"
