@@ -20,7 +20,7 @@ from office365.sharepoint.sites.sph_site import SPHSite
 from office365.sharepoint.sites.azure_container_Info import ProvisionedTemporaryAzureContainerInfo
 from office365.sharepoint.sites.upgrade_info import UpgradeInfo
 from office365.sharepoint.sites.usage_info import UsageInfo
-from office365.sharepoint.tenant.administration.site_administrators_info import SiteAdministratorsInfo
+from office365.sharepoint.tenant.administration.sites.administrators_info import SiteAdministratorsInfo
 from office365.sharepoint.usercustomactions.collection import UserCustomActionCollection
 from office365.sharepoint.webs.web import Web
 from office365.sharepoint.webs.template_collection import WebTemplateCollection
@@ -28,7 +28,14 @@ from office365.sharepoint.types.resource_path import ResourcePath as SPResPath
 
 
 class Site(BaseEntity):
-    """Represents a collection of sites in a Web application, including a top-level website and all its sub sites."""
+    """
+    Represents a collection of sites in a Web application, including a top-level website and all its sub sites.
+
+    A set of websites that are in the same content database, have the same owner, and share administration settings.
+    A site collection can be identified by a GUID or the URL of the top-level site for the site collection.
+    Each site collection contains a top-level site, can contain one or more subsites, and can have a shared
+    navigational structure.
+    """
 
     def __init__(self, context, resource_path=None):
         super(Site, self).__init__(context, ResourcePath("Site", resource_path))
@@ -91,6 +98,7 @@ class Site(BaseEntity):
         return ClientContext(url).site
 
     def get_available_tags(self):
+        """"""
         return_type = ClientResult(self.context, ClientValueCollection(ComplianceTag))
 
         def _site_loaded():
@@ -162,7 +170,8 @@ class Site(BaseEntity):
         :param office365.sharepoint.changes.query.ChangeQuery query: Specifies which changes to return
         """
         return_type = ChangeCollection(self.context)
-        qry = ServiceOperationQuery(self, "getChanges", None, query, "query", return_type)
+        payload = {"query": query}
+        qry = ServiceOperationQuery(self, "getChanges", None, payload, None, return_type)
         self.context.add_query(qry)
         return return_type
 
@@ -170,19 +179,24 @@ class Site(BaseEntity):
         """
         Returns a collection of recycle bin items based on the specified query.
 
-        :param int row_limit:
-        :param bool is_ascending:
+        :param int row_limit: The maximum number of Recycle Bin items to retrieve.
+        :param bool is_ascending: Specifies whether the Recycle Bin items are sorted in ascending order by the column
+            specified in the orderBy parameter. A value of true indicates ascending order, and a value of false
+            indicates descending order.
         """
-        result = RecycleBinItemCollection(self.context)
+        return_type = RecycleBinItemCollection(self.context)
         payload = {
             "rowLimit": row_limit,
             "isAscending": is_ascending
         }
-        qry = ServiceOperationQuery(self, "GetRecycleBinItems", None, payload, None, result)
+        qry = ServiceOperationQuery(self, "GetRecycleBinItems", None, payload, None, return_type)
         self.context.add_query(qry)
-        return result
+        return return_type
 
     def get_site_administrators(self):
+        """
+        Gets site collection administrators
+        """
         return_type = ClientResult(self.context, ClientValueCollection(SiteAdministratorsInfo))
 
         def _site_loaded():
@@ -582,6 +596,15 @@ class Site(BaseEntity):
         """
         return self.properties.get("ServerRelativePath", SPResPath())
 
+    @property
+    def status_bar_link(self):
+        """
+        Gets the status bar message link target for this site.
+
+        :rtype: str or None
+        """
+        return self.properties.get("StatusBarLink", None)
+
     @property
     def secondary_contact(self):
         """Gets or sets the secondary contact that is used for the site collection."""
@@ -646,7 +669,7 @@ class Site(BaseEntity):
 
     def set_property(self, name, value, persist_changes=True):
         if name == "__siteUrl":
-            super(Site, self).set_property("Url", value)
+            super(Site, self).set_property("Url", value, False)
             self._context = self.context.clone(value)
         else:
             super(Site, self).set_property(name, value, persist_changes)
