@@ -2,4 +2,4 @@ from office365.sharepoint.base_entity import BaseEntity
 
 
 class TeamSiteData(BaseEntity):
-    pass
+    """"""
