@@ -6,6 +6,7 @@ from office365.runtime.queries.service_operation import ServiceOperationQuery
 from office365.runtime.paths.resource_path import ResourcePath
 from office365.sharepoint.base_entity import BaseEntity
 from office365.sharepoint.base_entity_collection import BaseEntityCollection
+from office365.sharepoint.gtp.request_options import ChatGptRequestOptions
 from office365.sharepoint.listitems.collection import ListItemCollection
 from office365.sharepoint.lists.render_data_parameters import RenderListDataParameters
 from office365.sharepoint.lists.render_override_parameters import RenderListDataOverrideParameters
@@ -18,14 +19,16 @@ from office365.sharepoint.tenant.administration.insights.onedrive_site_sharing i
 from office365.sharepoint.tenant.administration.secondary_administrators_fields_data import \
     SecondaryAdministratorsFieldsData
 from office365.sharepoint.tenant.administration.secondary_administrators_info import SecondaryAdministratorsInfo
-from office365.sharepoint.tenant.administration.site_administrators_info import SiteAdministratorsInfo
-from office365.sharepoint.tenant.administration.site_creation_properties import SiteCreationProperties
-from office365.sharepoint.tenant.administration.site_properties import SiteProperties
-from office365.sharepoint.tenant.administration.site_properties_collection import SitePropertiesCollection
-from office365.sharepoint.tenant.administration.site_properties_enumerable_filter import SitePropertiesEnumerableFilter
+from office365.sharepoint.tenant.administration.sites.administrators_info import SiteAdministratorsInfo
+from office365.sharepoint.tenant.administration.sites.creation_properties import SiteCreationProperties
+from office365.sharepoint.tenant.administration.sites.properties import SiteProperties
+from office365.sharepoint.tenant.administration.sites.properties_collection import SitePropertiesCollection
+from office365.sharepoint.tenant.administration.sites.properties_enumerable_filter import SitePropertiesEnumerableFilter
 from office365.sharepoint.tenant.administration.siteinfo_for_site_picker import SiteInfoForSitePicker
 from office365.sharepoint.tenant.administration.spo_operation import SpoOperation
 from office365.sharepoint.tenant.administration.insights.top_files_sharing import TopFilesSharingInsights
+from office365.sharepoint.tenant.administration.webs.templates.collection import SPOTenantWebTemplateCollection
+from office365.sharepoint.tenant.settings import TenantSettings
 
 
 class Tenant(BaseEntity):
@@ -35,6 +38,24 @@ class Tenant(BaseEntity):
         static_path = ResourcePath("Microsoft.Online.SharePoint.TenantAdministration.Tenant")
         super(Tenant, self).__init__(context, static_path)
 
+    def get_corporate_catalog_site(self):
+        """Retrieves Corporate Catalog Site"""
+        settings = TenantSettings.current(self.context)
+        return_type = Site(self.context)
+
+        def _settings_loaded():
+            return_type.set_property("__siteUrl", settings.corporate_catalog_url)
+        settings.ensure_property("CorporateCatalogUrl", _settings_loaded)
+        return return_type
+
+    def get_chat_gpt_response(self):
+        """"""
+        return_type = ClientResult(self.context)
+        payload = {"requestOptions": ChatGptRequestOptions()}
+        qry = ServiceOperationQuery(self, "GetChatGptResponse", None, payload, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
     def delete_policy_definition(self, item_id):
         """
         :param int item_id:
@@ -51,6 +72,12 @@ class Tenant(BaseEntity):
         self.context.add_query(qry)
         return self
 
+    def get_spo_tenant_all_web_templates(self):
+        return_type = SPOTenantWebTemplateCollection(self.context)
+        qry = ServiceOperationQuery(self, "GetSPOTenantAllWebTemplates", None, None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
     def get_onedrive_site_sharing_insights(self, query_mode):
         return_type = ClientResult(self.context, OneDriveSiteSharingInsights())
         payload = {"queryMode": query_mode}
@@ -114,13 +141,16 @@ class Tenant(BaseEntity):
         self.context.add_query(qry)
         return return_type
 
-    def remove_home_site(self):
+    def remove_home_site(self, home_site_url):
         """
+        Remove home site
+
+        :param str home_site_url:
         """
-        return_type = ClientResult(self.context, str())
-        qry = ServiceOperationQuery(self, "RemoveSPHSite", None, None, None, return_type)
+        payload = {"homeSiteUrl": home_site_url}
+        qry = ServiceOperationQuery(self, "RemoveHomeSite", None, payload)
         self.context.add_query(qry)
-        return return_type
+        return self
 
     def has_valid_education_license(self):
         """"""
@@ -170,18 +200,26 @@ class Tenant(BaseEntity):
     def hub_sites(self, site_url):
         pass
 
+    def check_tenant_intune_license(self):
+        """
+        Checks whether a tenant has the Intune license.
+        """
+        return_type = ClientResult(self.context, bool())
+        qry = ServiceOperationQuery(self, "CheckTenantIntuneLicense", None, None, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
     def check_tenant_licenses(self, licenses):
         """
         Checks whether a tenant has the specified licenses.
 
         :param list[str] licenses: The list of licenses to check for.
-        :return:
         """
-        result = ClientResult(self.context)
+        return_type = ClientResult(self.context, bool())
         params = ClientValueCollection(str, licenses)
-        qry = ServiceOperationQuery(self, "CheckTenantLicenses", None, params, "licenses", result)
+        qry = ServiceOperationQuery(self, "CheckTenantLicenses", None, params, "licenses", return_type)
         self.context.add_query(qry)
-        return result
+        return return_type
 
     def get_site_status(self, url):
         """
@@ -229,12 +267,11 @@ class Tenant(BaseEntity):
         """
         Gets site collection administrators
 
-        :type site_id: str
+        :param str site_id: Site identifier
         """
         return_type = ClientResult(self.context, ClientValueCollection(SecondaryAdministratorsInfo))
-        payload = SecondaryAdministratorsFieldsData(site_id)
-        qry = ServiceOperationQuery(self, "GetSiteSecondaryAdministrators", None, payload,
-                                    "secondaryAdministratorsFieldsData", return_type)
+        payload = {"secondaryAdministratorsFieldsData": SecondaryAdministratorsFieldsData(site_id)}
+        qry = ServiceOperationQuery(self, "GetSiteSecondaryAdministrators", None, payload, None, return_type)
         self.context.add_query(qry)
         return return_type
 
@@ -246,9 +283,8 @@ class Tenant(BaseEntity):
         :type emails: list[str]
         :type site_id: str
         """
-        payload = SecondaryAdministratorsFieldsData(site_id, emails, names)
-        qry = ServiceOperationQuery(self, "SetSiteSecondaryAdministrators", None, payload,
-                                    "secondaryAdministratorsFieldsData", None)
+        payload = {"secondaryAdministratorsFieldsData": SecondaryAdministratorsFieldsData(site_id, emails, names)}
+        qry = ServiceOperationQuery(self, "SetSiteSecondaryAdministrators", None, payload, None, None)
         self.context.add_query(qry)
         return self
 
@@ -284,8 +320,8 @@ class Tenant(BaseEntity):
         :param str owner: Sets the login name of the owner of the new site.
         """
         return_type = SpoOperation(self.context)
-        payload = SiteCreationProperties(title=title, url=url, owner=owner)
-        qry = ServiceOperationQuery(self, "CreateSite", None, payload, "siteCreationProperties", return_type)
+        payload = {"siteCreationProperties": SiteCreationProperties(title=title, url=url, owner=owner)}
+        qry = ServiceOperationQuery(self, "CreateSite", None, payload, None, return_type)
         self.context.add_query(qry)
         return return_type
 
@@ -303,6 +339,7 @@ class Tenant(BaseEntity):
             """
             :type resp: requests.Response
             """
+            resp.raise_for_status()
             if items is None:
                 is_complete = op.is_complete
             else:
@@ -313,6 +350,7 @@ class Tenant(BaseEntity):
                 self.context.after_execute(_verify_site_status, items=items)
 
             return_type.set_property("__siteUrl", url)
+
         self.context.after_execute(_verify_site_status)
         return return_type
 
@@ -383,11 +421,8 @@ class Tenant(BaseEntity):
         :param str _filter:
         """
         return_type = SitePropertiesCollection(self.context)
-        qry = ServiceOperationQuery(self, "getSitePropertiesFromSharePointByFilters",
-                                    None,
-                                    SitePropertiesEnumerableFilter(_filter, start_index, include_detail),
-                                    "speFilter",
-                                    return_type)
+        payload = {"speFilter": SitePropertiesEnumerableFilter(_filter, start_index, include_detail)}
+        qry = ServiceOperationQuery(self, "getSitePropertiesFromSharePointByFilters", None, payload, None, return_type)
         self.context.add_query(qry)
         return return_type
 
