@@ -1,4 +1,4 @@
-from office365.runtime.queries.service_operation_query import ServiceOperationQuery
+from office365.runtime.queries.service_operation import ServiceOperationQuery
 from office365.runtime.paths.service_operation import ServiceOperationPath
 from office365.sharepoint.base_entity_collection import BaseEntityCollection
 from office365.sharepoint.webhooks.subscription import Subscription
@@ -15,20 +15,26 @@ class SubscriptionCollection(BaseEntityCollection):
         """Gets the subscription with the specified ID."""
         return Subscription(self.context, ServiceOperationPath("getById", [_id], self.resource_path))
 
-    def add(self, information_or_notification):
+    def add(self, parameters):
         """
-        :type information_or_notification: SubscriptionInformation or str
+        :param SubscriptionInformation or str parameters: Subscription information object or notification string
         """
         return_type = Subscription(self.context)
         self.add_child(return_type)
-        if isinstance(information_or_notification, SubscriptionInformation):
-            qry = ServiceOperationQuery(self, "Add", None, information_or_notification, "parameters", return_type)
+
+        def _create_and_add_query(information):
+            """
+            :type information: SubscriptionInformation
+            """
+            payload = {"parameters": information}
+            qry = ServiceOperationQuery(self, "Add", None, payload, None, return_type)
             self.context.add_query(qry)
+
+        if isinstance(parameters, SubscriptionInformation):
+            _create_and_add_query(parameters)
         else:
             def _parent_loaded():
-                information = SubscriptionInformation(information_or_notification, self._parent.properties["Id"])
-                next_qry = ServiceOperationQuery(self, "Add", None, information, "parameters", return_type)
-                self.context.add_query(next_qry)
+                _create_and_add_query(SubscriptionInformation(parameters, self._parent.properties["Id"]))
             self._parent.ensure_property("Id", _parent_loaded)
         return return_type
 
