@@ -2,6 +2,7 @@
 
 from office365.runtime.client_result import ClientResult
 from office365.runtime.client_value_collection import ClientValueCollection
+from office365.runtime.queries.function import FunctionQuery
 from office365.runtime.queries.service_operation import ServiceOperationQuery
 from office365.runtime.paths.resource_path import ResourcePath
 from office365.runtime.paths.service_operation import ServiceOperationPath
@@ -25,11 +26,14 @@ from office365.sharepoint.flows.synchronization_result import FlowSynchronizatio
 from office365.sharepoint.folders.folder import Folder
 from office365.sharepoint.folders.collection import FolderCollection
 from office365.sharepoint.listitems.listitem import ListItem
+from office365.sharepoint.lists.creation_information import ListCreationInformation
 from office365.sharepoint.lists.document_library_information import DocumentLibraryInformation
 from office365.sharepoint.lists.get_parameters import GetListsParameters
 from office365.sharepoint.lists.list import List
 from office365.sharepoint.lists.collection import ListCollection
 from office365.sharepoint.lists.template_collection import ListTemplateCollection
+from office365.sharepoint.lists.template_type import ListTemplateType
+from office365.sharepoint.marketplace.corporatecuratedgallery.available_addins_response import SPAvailableAddinsResponse
 from office365.sharepoint.navigation.navigation import Navigation
 from office365.sharepoint.permissions.base_permissions import BasePermissions
 from office365.sharepoint.permissions.roles.definitions.collection import RoleDefinitionCollection
@@ -46,6 +50,7 @@ from office365.sharepoint.sharing.links.access_request import SharingLinkAccessR
 from office365.sharepoint.sharing.object_sharing_settings import ObjectSharingSettings
 from office365.sharepoint.sharing.links.data import SharingLinkData
 from office365.sharepoint.sharing.result import SharingResult
+from office365.sharepoint.sharing.shared_document_info import SharedDocumentInfo
 from office365.sharepoint.sitescripts.utility import SiteScriptUtility
 from office365.sharepoint.marketplace.sitecollection.appcatalog.accessor import SiteCollectionCorporateCatalogAccessor
 from office365.sharepoint.marketplace.tenant.appcatalog.accessor import TenantCorporateCatalogAccessor
@@ -59,6 +64,7 @@ from office365.sharepoint.webparts.client.collection import ClientWebPartCollect
 from office365.sharepoint.webs.calendar_type import CalendarType
 from office365.sharepoint.webs.context_web_information import ContextWebInformation
 from office365.sharepoint.fields.datetime_field_format_type import DateTimeFieldFormatType
+from office365.sharepoint.webs.modernize_homepage_result import ModernizeHomepageResult
 from office365.sharepoint.webs.multilingual_settings import MultilingualSettings
 from office365.sharepoint.webs.regional_settings import RegionalSettings
 from office365.sharepoint.sitescripts.serialization_info import SiteScriptSerializationInfo
@@ -70,8 +76,13 @@ from office365.sharepoint.webs.theme_info import ThemeInfo
 
 
 class Web(SecurableObject):
-    """Represents a SharePoint site. A site is a type of SP.SecurableObject.
-    Refer this link https://msdn.microsoft.com/en-us/library/office/dn499819.aspx for a details"""
+    """
+    Represents a SharePoint site. A site is a type of SecurableObject.
+
+    A group of related webpages that is hosted by a server on the World Wide Web or an intranet.
+    Each website has its own entry points, metadata, administration settings, and workflows.
+    Also referred to as web site.
+    """
 
     def __init__(self, context, resource_path=None):
         """
@@ -85,6 +96,26 @@ class Web(SecurableObject):
         super(Web, self).__init__(context, resource_path)
         self._web_url = None
 
+    def add_list(self, title, template_type=ListTemplateType.GenericList):
+        """
+        Creates a new list and adds it to the web.
+
+        :param str title: Specifies the display name of the new list.
+        :param int template_type: Specifies the list server template of the new list.
+        """
+        info = ListCreationInformation(title, None, template_type)
+        return self.lists.add(info)
+
+    def available_addins(self, server_relative_urls=None):
+        """
+        :param list[str] server_relative_urls:
+        """
+        payload = {"serverRelativeUrls": server_relative_urls}
+        return_type = ClientResult(self.context, SPAvailableAddinsResponse())
+        qry = ServiceOperationQuery(self, "AvailableAddins", None, payload, None, return_type)
+        self.context.add_query(qry)
+        return return_type
+
     def add_cross_farm_message(self, message):
         """
         :param str message:
@@ -117,6 +148,7 @@ class Web(SecurableObject):
         return result
 
     def consent_to_power_platform(self):
+        """"""
         return_type = FlowSynchronizationResult(self.context)
         qry = ServiceOperationQuery(self, "ConsentToPowerPlatform", None, None, None, return_type)
         self.context.add_query(qry)
@@ -171,23 +203,25 @@ class Web(SecurableObject):
         """
         return_type = PushNotificationSubscriberCollection(self.context)
 
-        def _create_query(login_name):
-            return ServiceOperationQuery(self, "GetPushNotificationSubscribersByUser",
-                                         [login_name], None, None, return_type)
+        def _create_and_add_query(login_name):
+            """
+            :type login_name: str
+            """
+            qry = ServiceOperationQuery(self, "GetPushNotificationSubscribersByUser",
+                                        [login_name], None, None, return_type)
+            self.context.add_query(qry)
 
         if isinstance(user, User):
             def _user_loaded():
-                next_qry = _create_query(user.login_name)
-                self.context.add_query(next_qry)
+                _create_and_add_query(user.login_name)
 
             user.ensure_property("LoginName", _user_loaded)
         else:
-            qry = _create_query(user)
-            self.context.add_query(qry)
+            _create_and_add_query(user)
         return return_type
 
     @staticmethod
-    def create_organization_sharing_link(context, url, is_edit_link):
+    def create_organization_sharing_link(context, url, is_edit_link=False):
         """ Creates and returns an organization-internal link that can be used to access a document and gain permissions
            to it.
 
@@ -196,7 +230,7 @@ class Web(SecurableObject):
             string parameters, forSharing set to 1 if sharing, and bypass set to 1 to bypass any mobile logic.
         :param bool is_edit_link: If true, the link will allow the logged in user to edit privileges on the item.
         """
-        return_type = ClientResult(context)
+        return_type = ClientResult(context, str())
         params = {"url": url, "isEditLink": is_edit_link}
         qry = ServiceOperationQuery(context.web, "CreateOrganizationSharingLink", None, params, None, return_type)
         qry.static = True
@@ -243,14 +277,14 @@ class Web(SecurableObject):
         :type context: office365.sharepoint.client_context.ClientContext
         :param str page_full_url: Specifies the URL from which to return the site URL.
         """
-        result = ClientResult(context)
+        return_type = ClientResult(context, str())
         payload = {
             "pageFullUrl": page_full_url
         }
-        qry = ServiceOperationQuery(context.web, "GetWebUrlFromPageUrl", None, payload, None, result)
+        qry = ServiceOperationQuery(context.web, "GetWebUrlFromPageUrl", None, payload, None, return_type)
         qry.static = True
         context.add_query(qry)
-        return result
+        return return_type
 
     def create_default_associated_groups(self, user_login, user_login2, group_name_seed):
         """
@@ -296,7 +330,6 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return return_type
 
-
     def sync_flow_instances(self, target_web_url):
         """
         :param str target_web_url:
@@ -318,6 +351,7 @@ class Web(SecurableObject):
         return return_type
 
     def get_all_client_side_components(self):
+        """"""
         return_type = ClientResult(self.context, str())
         qry = ServiceOperationQuery(self, "GetAllClientSideComponents", None, None, None, return_type)
         self.context.add_query(qry)
@@ -362,10 +396,13 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return self
 
-    def get_lists(self):
+    def get_lists(self, row_limit=100):
+        """
+        :param int row_limit: Specifies a limit for the number of lists in the query that are returned per page
+        """
         return_type = ListCollection(self.context)
         payload = {
-            "getListsParams": GetListsParameters()
+            "getListsParams": GetListsParameters(row_limit=row_limit)
         }
         qry = ServiceOperationQuery(self, "GetLists", None, payload, None, return_type)
         self.context.add_query(qry)
@@ -438,10 +475,10 @@ class Web(SecurableObject):
 
         :param str decoded_url: Contains the site-relative path for a list, for example, /Lists/Announcements.
         """
-        safe_decoded_url = self.context.create_safe_url(decoded_url)
+        path = SPResPath.create_relative(self.context.base_url, decoded_url)
         return_type = List(self.context)
         self.lists.add_child(return_type)
-        qry = ServiceOperationQuery(self, "GetListUsingPath", SPResPath(safe_decoded_url), None, None, return_type)
+        qry = ServiceOperationQuery(self, "GetListUsingPath", path, None, None, return_type)
         self.context.add_query(qry)
         return return_type
 
@@ -501,11 +538,11 @@ class Web(SecurableObject):
 
         :type context: office365.sharepoint.client_context.ClientContext
         """
-        result = ClientResult(context, str())
-        qry = ServiceOperationQuery(context.web, "GetContextWebThemeData", None, None, None, result)
+        return_type = ClientResult(context, str())
+        qry = ServiceOperationQuery(context.web, "GetContextWebThemeData", None, None, None, return_type)
         qry.static = True
         context.add_query(qry)
-        return result
+        return return_type
 
     def create_site_page(self, page_metadata):
         """Create a site page
@@ -519,26 +556,27 @@ class Web(SecurableObject):
         return return_type
 
     @staticmethod
-    def create_anonymous_link(context, url, is_edit_link):
+    def create_anonymous_link(context, url, is_edit_link, return_type=None):
         """Create an anonymous link which can be used to access a document without needing to authenticate.
 
         :param bool is_edit_link: If true, the link will allow the guest user edit privileges on the item.
         :param str url: The URL of the site, with the path of the object in SharePoint represented as query
         string parameters
         :param office365.sharepoint.client_context.ClientContext context: client context
+        :param ClientResult return_type: Return type
         """
-        return_type = ClientResult(context, str())
+        if return_type is None:
+            return_type = ClientResult(context, str())
         payload = {
-            "url": context.create_safe_url(url, False),
+            "url": str(SPResPath.create_absolute(context.base_url, url)),
             "isEditLink": is_edit_link
         }
-        qry = ServiceOperationQuery(context.web, "CreateAnonymousLink", None, payload, None, return_type)
-        qry.static = True
+        qry = ServiceOperationQuery(context.web, "CreateAnonymousLink", None, payload, None, return_type, True)
         context.add_query(qry)
         return return_type
 
     @staticmethod
-    def create_anonymous_link_with_expiration(context, url, is_edit_link, expiration_string):
+    def create_anonymous_link_with_expiration(context, url, is_edit_link, expiration_string, return_type=None):
         """
         Creates and returns an anonymous link that can be used to access a document without needing to authenticate.
 
@@ -550,10 +588,12 @@ class Web(SecurableObject):
         anonymous link. Both the minutes and hour value MUST be specified for the difference between the local and
         UTC time. Midnight is represented as 00:00:00.
         :param office365.sharepoint.client_context.ClientContext context: client context
+        :param ClientResult return_type: Return type
         """
-        return_type = ClientResult(context, str())
+        if return_type is None:
+            return_type = ClientResult(context, str())
         payload = {
-            "url": context.create_safe_url(url, False),
+            "url": str(SPResPath.create_absolute(context.base_url, url)),
             "isEditLink": is_edit_link,
             "expirationString": expiration_string
         }
@@ -606,21 +646,26 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return return_type
 
-    def get_file_by_server_relative_url(self, url):
+    def get_file_by_server_relative_url(self, server_relative_url):
         """
-        Returns the file object located at the specified server-relative URL.
+        Returns the file object located at the specified server-relative URL, for example:
+            - "/sites/MySite/Shared Documents/MyDocument.docx"
+            - "Shared Documents/MyDocument.docx"
 
-        :param str url: Specifies the server-relative URL for the file.
+        :param str server_relative_url: Specifies the server-relative URL for the file.
         """
-        return File(self.context, ServiceOperationPath("getFileByServerRelativeUrl", [url], self.resource_path))
+        path = SPResPath.create_relative(self.context.base_url, server_relative_url)
+        return File(self.context, ServiceOperationPath("getFileByServerRelativeUrl", [str(path)], self.resource_path))
 
-    def get_file_by_server_relative_path(self, decoded_url):
-        """Returns the file object located at the specified server-relative path.
-        Prefer this method over get_folder_by_server_relative_url since it supports % and # symbols in names
+    def get_file_by_server_relative_path(self, path):
+        """Returns the file object located at the specified server-relative path, for example:
+            - "/sites/MySite/Shared Documents/MyDocument.docx"
+            - "Shared Documents/MyDocument.docx"
+        Note: prefer this method over get_folder_by_server_relative_url since it supports % and # symbols in names
 
-        :param str or SPResPath decoded_url: Contains the server-relative path of the file.
+        :param str path: Contains the server-relative path of the file.
         """
-        path = decoded_url if isinstance(decoded_url, SPResPath) else SPResPath(decoded_url)
+        path = SPResPath.create_relative(self.context.base_url, path)
         return File(self.context,
                     ServiceOperationPath("getFileByServerRelativePath", path.to_json(), self.resource_path))
 
@@ -632,10 +677,12 @@ class Web(SecurableObject):
         return Folder(self.context, ServiceOperationPath("getFolderByServerRelativeUrl", [url], self.resource_path))
 
     def get_folder_by_server_relative_path(self, decoded_url):
-        """Returns the folder object located at the specified server-relative URL.
+        """Returns the folder object located at the specified server-relative URL, for example:
+             - "/sites/MySite/Shared Documents"
+             - "Shared Documents"
         Prefer this method over get_folder_by_server_relative_url since it supports % and # symbols
 
-        :param str decoded_url: Contains the server-relative URL for the folder.
+        :param str decoded_url: Contains the server-relative URL for the folder
         """
         path = SPResPath(decoded_url)
         return Folder(self.context,
@@ -647,7 +694,7 @@ class Web(SecurableObject):
 
         :param str path: relative server URL (path) to a folder
         """
-        return self.root_folder.folders.ensure_folder_path(path)
+        return self.root_folder.folders.ensure_path(path)
 
     def ensure_user(self, login_name):
         """Checks whether the specified logon name belongs to a valid user of the website, and if the logon name does
@@ -676,10 +723,18 @@ class Web(SecurableObject):
 
         :param str user_name: Specifies the user login name.
         """
-        result = ClientResult(self.context, BasePermissions())
-        qry = ServiceOperationQuery(self, "GetUserEffectivePermissions", [user_name], None, None, result)
+        return_type = ClientResult(self.context, BasePermissions())
+        qry = ServiceOperationQuery(self, "GetUserEffectivePermissions", [user_name], None, None, return_type)
         self.context.add_query(qry)
-        return result
+        return return_type
+
+    def get_list_by_title(self, title):
+        """
+        Returns the list with the specified display name.
+
+        :param str title: Specifies the display name
+        """
+        return List(self.context, ServiceOperationPath("GetListByTitle", [title], self.resource_path))
 
     def does_user_have_permissions(self, permission_mask):
         """Returns whether the current user has the given set of permissions.
@@ -728,8 +783,8 @@ class Web(SecurableObject):
 
         :param str path: A string that contains the site-relative URL for a list, for example, /Lists/Announcements.
         """
-        safe_path = self.context.create_safe_url(path)
-        return List(self.context, ServiceOperationPath("getList", [safe_path], self.resource_path))
+        safe_path = SPResPath.create_relative(self.context.base_url, path)
+        return List(self.context, ServiceOperationPath("getList", [str(safe_path)], self.resource_path))
 
     def get_changes(self, query):
         """Returns the collection of all changes from the change log that have occurred within the scope of the site,
@@ -738,7 +793,8 @@ class Web(SecurableObject):
         :param office365.sharepoint.changes.query.ChangeQuery query: Specifies which changes to return
         """
         return_type = ChangeCollection(self.context)
-        qry = ServiceOperationQuery(self, "getChanges", None, query, "query", return_type)
+        payload = {"query": query}
+        qry = ServiceOperationQuery(self, "getChanges", None, payload, None, return_type)
         self.context.add_query(qry)
         return return_type
 
@@ -1221,12 +1277,11 @@ class Web(SecurableObject):
         Returns the list item that is associated with the specified server-relative path.
 
         :param str decoded_url: A string that contains the server-relative path,
-        for example, "/sites/MySite/Shared Documents/MyDocument.docx".
+        for example, "/sites/MySite/Shared Documents/MyDocument.docx" or "Shared Documents/MyDocument.docx".
         :return: ListItem
         """
-        path = SPResPath(self.context.create_safe_url(decoded_url))
-        return ListItem(self.context,
-                        ServiceOperationPath("GetListItemUsingPath", path, self.resource_path))
+        params = SPResPath.create_relative(self.context.base_url, decoded_url)
+        return ListItem(self.context, ServiceOperationPath("GetListItemUsingPath", params, self.resource_path))
 
     def get_catalog(self, type_catalog):
         """Gets the list template gallery, site template gallery, or Web Part gallery for the Web site.
@@ -1327,6 +1382,12 @@ class Web(SecurableObject):
         self.context.add_query(qry)
         return self
 
+    def sync_hub_site_theme(self):
+        """"""
+        qry = ServiceOperationQuery(self, "SyncHubSiteTheme")
+        self.context.add_query(qry)
+        return self
+
     def assign_document_id(self, site_prefix, enabled=True):
         """
         Assign Document IDs
@@ -1471,6 +1532,12 @@ class Web(SecurableObject):
         return self.properties.get('Lists',
                                    ListCollection(self.context, ResourcePath("lists", self.resource_path)))
 
+    @property
+    def onedrive_shared_items(self):
+        return self.properties.get('OneDriveSharedItems',
+                                   BaseEntityCollection(self.context, SharedDocumentInfo,
+                                                        ResourcePath("OneDriveSharedItems", self.resource_path)))
+
     @property
     def site_users(self):
         """
@@ -1515,6 +1582,13 @@ class Web(SecurableObject):
         return self.properties.get('AssociatedMemberGroup',
                                    Group(self.context, ResourcePath("AssociatedMemberGroup", self.resource_path)))
 
+    @property
+    def can_modernize_homepage(self):
+        """Specifies the site theme associated with the site"""
+        return self.properties.get('CanModernizeHomepage',
+                                   ModernizeHomepageResult(self.context,
+                                                           ResourcePath("CanModernizeHomepage", self.resource_path)))
+
     @property
     def fields(self):
         """Specifies the collection of all the fields (2) in the site (2)."""
@@ -1820,6 +1894,7 @@ class Web(SecurableObject):
                 "HostedApps": self.hosted_apps,
                 "ListTemplates": self.list_templates,
                 "MultilingualSettings": self.multilingual_settings,
+                "OneDriveSharedItems": self.onedrive_shared_items,
                 "ParentWeb": self.parent_web,
                 "PushNotificationSubscribers": self.push_notification_subscribers,
                 "RootFolder": self.root_folder,
